#include "cleardataform.h"

#include <qvariant.h>
#include <qimage.h>
#include <qpixmap.h>

#include "ebtdefines.h"

/*
 *  Constructs a ClearDataForm as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  true to construct a modal dialog.
 */

ClearDataForm::ClearDataForm(QWidget* parent, bool modal, Qt::WindowFlags fl)
    : QDialog(parent, fl)
{
    setupUi(this);
    setModal(modal);

    init();
}

/*
 *  Destroys the object and frees any allocated resources
 */
ClearDataForm::~ClearDataForm()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void ClearDataForm::languageChange()
{
    retranslateUi(this);
}

void ClearDataForm::init()
{
    checkBoxAllData->setChecked(TRUE);
    connect(this, SIGNAL(clearData(int, int)), parentWidget(), SLOT(clearData(int, int)));
}

void ClearDataForm::setAllDataChecked(bool on)
{
    if(on)
    {
	checkBoxOutput->setChecked(TRUE);
	checkBoxOutput->setEnabled(FALSE);
	checkBoxComputed->setChecked(TRUE);
	checkBoxComputed->setEnabled(FALSE);
	checkBoxFixed->setChecked(TRUE);
	checkBoxFixed->setEnabled(FALSE);
	checkBoxState->setChecked(TRUE);
	checkBoxState->setEnabled(FALSE);
    }
    else
    {
	checkBoxOutput->setEnabled(TRUE);
	checkBoxComputed->setEnabled(TRUE);
	checkBoxFixed->setEnabled(TRUE);
	checkBoxState->setEnabled(TRUE);
    }
}

void ClearDataForm::clearData()
{
    int fromMemory = 0, fromFile = 0;
    if(checkBoxOutput->isChecked())
	fromMemory |= OUTPUTVALUES;
    if(checkBoxComputed->isChecked())
	fromMemory |= COMPUTEDVALUES;
    if(checkBoxFixed->isChecked())
	fromMemory |= FIXEDVALUES;
    if(checkBoxState->isChecked())
	fromMemory |= STATEOUTPUT;
    if(checkBoxOutputFile->isChecked())
	fromFile |= OUTPUTVALUES;
    if(checkBoxComputedFile->isChecked())
	fromFile |= COMPUTEDVALUES;
    if(checkBoxFixedFile->isChecked())
	fromFile |= FIXEDVALUES;
    if(checkBoxStateFile->isChecked())
	fromFile |= STATEOUTPUT;
    
    emit clearData(fromMemory, fromFile);
    accept();
}

void ClearDataForm::setAllFilesChecked( bool on )
{

    if(on)
    {
	checkBoxOutputFile->setChecked(TRUE);
	checkBoxOutputFile->setEnabled(FALSE);
	checkBoxComputedFile->setChecked(TRUE);
	checkBoxComputedFile->setEnabled(FALSE);
	checkBoxFixedFile->setChecked(TRUE);
	checkBoxFixedFile->setEnabled(FALSE);
	checkBoxStateFile->setChecked(TRUE);
	checkBoxStateFile->setEnabled(FALSE);
    }
    else
    {
	checkBoxOutputFile->setEnabled(TRUE);
	checkBoxComputedFile->setEnabled(TRUE);
	checkBoxFixedFile->setEnabled(TRUE);
	checkBoxStateFile->setEnabled(TRUE);
    }
}
