#ifndef EBTCSBFILE_H
#define EBTCSBFILE_H

#include "ebtfile.h"
#include "ebtdefidcard.h"
#include "ebtdefout.h"
#include <QMap>
#include <QDateTime>

// List of magic keys of all recognized CSB file types
const uint CSB_MAGIC_KEY1 = 20030509;

class EBTCSBFile : public EBTFile
{
    void fileCorrupt();

    typedef struct staterecord {
	double timeVal;
	long int filePos;
	uint numPopulations;
	int maxCohorts;
	bool read;
	QMap<uint, QString> description;
	QMap<uint, uint> popCohorts;
	QMap<uint, uint> popColumns;
    } stateRecord;
    QMap<uint, stateRecord> stateRecordMap;
    char *columnData;
    
    EBTDefIDcard *defidcard;
    double **popIDcard;
    uint popIDcardSize;
    int currentPopIDcard;
    int newConstDim;

    EBTDefOut *defoutput;
    double **computedOutput;
    int compOutputSize;
    int outputVarNr;

    bool scanOK;
    uint times;
    double *csbPar;
    QDateTime lastModified;
    QFile *file;
    
public:
    EBTCSBFile();
    ~EBTCSBFile();
    bool open();
    void unload();
    bool scanSucceeded(){return scanOK;};
    bool checkFile();
    bool fileChanged();
    void scan();
    bool readState(uint recordNr);
    void print();
    uint numStates();
    double timeVal(uint stateNr);
    uint numPopulations(uint stateNr);
    uint numColumns(uint stateNr, uint popNr);
    uint numCohorts(uint stateNr, uint popNr);
    QString description(uint stateNr, uint popNr);
    int computeIDcard(uint stateNr);
    int initComputeOutput();
    void computeOutput(uint stateNr);
    void column(uint stateNr, uint popNr, uint xCol, uint yCol, double* colPtr[2]);
    double *compOutput(uint stateNr){return computedOutput[stateNr];};
    uint outputVarNum(){return outputVarNr;};
    void setDefID(EBTDefIDcard *defid);
    void setDefOut(EBTDefOut *defout);
    void clearColumnData();
    void clearIDcard();
    void clearCompOutput();
    QString exportState(uint stateNr);
};
	
#endif	// EBTCSBFILE_H
