#include <float.h>
#include <qfileinfo.h>
#include <qstringlist.h>
#include <QTextStream>
#include <qmessagebox.h>
#include "ebtdatafile.h"

EBTDataFile::EBTDataFile()
{
    setTemplateFile(QString());
}

EBTDataFile::~EBTDataFile()
{
}

void EBTDataFile::load()
{
    buffer = QString();
    
    if(fileName().isEmpty())
	return;
    
    QFile file(absFilePath());
    
    if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
	QTextStream stream(&file);
	buffer = stream.readAll();
    }
    file.close();
    setChanged(FALSE);
}


EBTDataMap EBTDataFile::loadData()
{
  bool        first = TRUE;
  QString     str, tmpStr;
  QStringList strList;
  EBTDataMap  data;
  uint        rowNum = 0, colNum = 0, errorCount = 0;
  nRows = 0;

  QFile file(absFilePath());

  if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
      QTextStream stream(&file);
      while ((str = stream.readLine()) != QString())
        {
          if (str.startsWith("#")) continue;
          if (str.startsWith("@")) continue;
          if (str.length() > 1)
            {
              tmpStr  = str.replace(QChar('\n'), " ").replace(QChar('\t'), " ");
              strList = tmpStr.split(" ", Qt::SkipEmptyParts);

              if (first)
                {
                  nCols  = 0;
                  rowNum = 0;
                  nCols  = strList.count();

                  if (nCols < 1 || nCols > MAXCOLUMNS)
                    {
                      QMessageBox::warning(0, QString(EBTCAPTION + "Error"), QString("Column count incorrect"), QMessageBox::Ok);
                      data.clear();
                      return data;                                                   //??
                    }

                  for (colNum = 0; colNum < nCols; colNum++)
                    {
                      EBTData dataVec;
                      data[colNum] = dataVec;
                    }

                }                                                                    // if(first)

              uint nColsRead = strList.count();
              if ((nColsRead != nCols) && (!errorCount))
                {
                  QMessageBox::warning(0, QString(EBTCAPTION + "Warning"), QString("Number of columns not consistent"), QMessageBox::Ok);
                  errorCount++;
                }

              for (colNum = 0; colNum < nCols; colNum++)
                {
                  if (colNum < nColsRead)
                    data[colNum].appendValue(strList[colNum].toDouble());
                  else
                    data[colNum].appendValue(DBL_MAX);
                }
              // toDouble() will return 0.0 when conversion fails

              rowNum++;
              first = FALSE;
            }                                                                        // if(str.length() > 1)
        }                                                                            // while readline
    }
  nRows = rowNum;
  file.close();
  return data;
}

EBTDataMap EBTDataFile::appendData(EBTDataMap data)
{
  QString     str, tmpStr;
  QStringList strList;
  uint        rowNum, colNum, errorCount = 0;

  QFile file(absFilePath());
  rowNum = data.count();
  nCols  = data.keys().size();
  if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
      QTextStream stream(&file);
      while ((str = stream.readLine()) != QString())
        {
          if (str.startsWith("#")) continue;
          if (str.startsWith("@")) continue;
          if (str.length() > 1)
            {
              tmpStr  = str.replace(QChar('\n'), " ").replace(QChar('\t'), " ");
              strList = tmpStr.split(" ", Qt::SkipEmptyParts);

              if ((strList.count() != (int)nCols) && (!errorCount))
                {
                  QMessageBox::warning(0, QString(EBTCAPTION + "Warning"), QString("Number of columns not consistent"), QMessageBox::Ok);
                  errorCount++;
                }

              for (colNum = 0; colNum < nCols; colNum++) data[colNum].appendValue(strList[colNum].toDouble());
              // toDouble() will return 0.0 when conversion fails

              rowNum++;
            }
        }                                                                            // while readline
    }
  nRows = rowNum;
  file.close();
  return data;
}

void EBTDataFile::fillNums()
{
    bool first = TRUE;
    QString str, tmpStr;
    QStringList strList;
    nCols = nRows = 0;
    
    QFile file(absFilePath());
    
    if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
	QTextStream stream(&file);
	while ((str = stream.readLine()) != QString())
	{
	    if (str.startsWith("#")) continue;
	    if (str.startsWith("@")) continue;
	    {
		if (str.length() > 1)
		{
		    if(first)
		    {
			nRows = 0;
			tmpStr = str.replace(QChar('\n'), " ").replace(QChar('\t'), " ");
			strList = tmpStr.split(" ", Qt::SkipEmptyParts);
			nCols = strList.count();
		    }
		    nRows++;
		    first = FALSE;
		}
	    }
	}
    }
    file.close(); 
}

