#ifndef EBTDATASET_H
#define EBTDATASET_H

#include "qwt_symbol.h"
#include "qwt_plot.h"
#include "ebtplotcurve.h"
#include <QVector>
#include "ebtdata.h"

class EBTDataSet 
{
private:
    uint xcol;
    uint ycol;
    EBTData xdata;
    EBTData ydata;
    
    int datatype;
    int populationnr;
    QwtPlot::Axis xaxis;
    QwtPlot::Axis yaxis;
    QString legendlabel;
    QwtSymbol *curvesymbol;
    QPen linepen;
    QwtPlotCurve::CurveStyle curvestyle;
    int histnumbars;
    EBTPlotCurve *curveid;
    int oldsize;
    bool isnew;
    
public:
    EBTDataSet();
    ~EBTDataSet(){};
    
    void clear();
    void xAppend(double value){ xdata.appendValue(value); };
    void yAppend(double value){ ydata.appendValue(value); };
    
    uint xCol(){ return xcol; };
    uint yCol(){ return ycol; };
    QVector<double> xData(){ return xdata.data(); };
    QVector<double> yData(){ return ydata.data(); };
    double * xDataPnt(){ return xdata.dataPnt(); };
    double * yDataPnt(){ return ydata.dataPnt(); };
    double xMinVal(){ return xdata.minVal(); };
    double xMaxVal(){ return xdata.maxVal(); };
    double yMinVal(){ return ydata.minVal(); };
    double yMaxVal(){ return ydata.maxVal(); };
    int size(){ return xdata.size(); };
    
    int dataType(){ return datatype; };
    int populationNr(){ return populationnr; };
    QwtPlot::Axis xAxis(){ return xaxis; };
    QwtPlot::Axis yAxis(){ return yaxis; };
    QString legendLabel(){ return legendlabel; };
    QwtSymbol *curveSymbol(){ return curvesymbol; };
    QPen linePen(){ return linepen; };
    QwtPlotCurve::CurveStyle curveStyle(){ return curvestyle; };
    int histNumBars(){ return histnumbars; };
    EBTPlotCurve *curveID(){ return curveid; };
    int oldSize(){ return oldsize; };
    bool isNew(){ return isnew; };
    
    void setXCol(uint value){ xcol = value; };
    void setYCol(uint value){ ycol = value; };
    void setXData(const EBTData &value){ xdata = value; };
    void setYData(const EBTData &value){ ydata = value; };
    void setDataType(int value){ datatype = value; };
    void setPopulationNr(int value){ populationnr = value; };
    void setXAxis(QwtPlot::Axis value){ xaxis = value; };
    void setYAxis(QwtPlot::Axis value){ yaxis = value; };
    void setLegendLabel(QString value){ legendlabel = value; };
    void setCurveSymbol(QwtSymbol *value);
    void setHistNumBars(int value){ histnumbars = value; };
    void setLinePen(QPen value){ linepen = value; };
    void setCurveStyle(QwtPlotCurve::CurveStyle value){ curvestyle = value; };
    void setCurveID(EBTPlotCurve * value){ curveid = value; };
    void setOldSize(int value){ oldsize = value; };
    void setIsNew( bool value){ isnew = value;};
};

typedef QMap<uint, EBTDataSet> EBTDataSetMap;

#endif // EBTDATASET_H
