#ifndef EBTDEFINES_H
#define EBTDEFINES_H

#include <qstringlist.h>

#ifndef uint
typedef unsigned int uint;
#endif
typedef char* charPtr;
typedef double* doublePtr;

enum fileType{EBTRUN, EBTOGR, EBTSGR, EBTDEFID, EBTDEFOUT, EBTMODEL, EBTPR};

const uint MAXCOLUMNS = 100;
const uint MAXTIME = 10000;
//const uint BUFSIZE = 50000;
//const QString EBTVERSION("0.5.0, April 12, 2006");
const QString EBTVERSION(__DATE__);
const QString EBTCAPTION("EBTtool -- ");
const QString OUTPUTDESC("Output graph");
const QString STATEDESC("State graph");
const QString colWarning("Column number in data set definition out of range!");
const QString popWarning("Population number in data set definition out of range!");

const QString clangCC("clang++");
const QString gnuCC("g++");
const QString intelCC("icpc");
const QString MingwCC("g++");
const QString MSVC("cl");

const QString defModelCCFlagsClangCC = "-O2";
const QString defOtherCCFlagsClangCC = "-O2 -g -fPIC";
const QString defLDFlagsClangCC = "-lm";

const QString defModelCCFlagsGnuCC = "-O2";
const QString defOtherCCFlagsGnuCC = "-O2 -g -fPIC";
const QString defLDFlagsGnuCC = "-lm";

const QString defModelCCFlagsIntelCC = "-fast";
const QString defOtherCCFlagsIntelCC = "-fast -g -fPIC";
const QString defLDFlagsIntelCC = "-lm";

const QString defModelCCFlagsMingwCC = "-O2";
const QString defOtherCCFlagsMingwCC = "-O2 -g";
const QString defLDFlagsMingwCC = "-lm";

const QString defModelCCFlagsMSVC = "/TP /EHsc /nologo";
const QString defOtherCCFlagsMSVC = "/TP /EHsc /nologo";
const QString defLDFlagsMSVC = "/nologo";

#if (defined(Q_OS_WIN) && defined(_MSC_VER))

// When compiled with MSVC, only allow MSVC as module compiler
const QString defCompiler = MSVC;
const QStringList compilerList = (QStringList() << MSVC);
const QString defTmpPath = "C:/Windows/Temp";

#if !defined(NAN)
const unsigned long nan[2] = {0xffffffff, 0x7fffffff};
#define NAN 	(*(const double *)nan)
#define isnan	_isnan
#endif

#elif (defined(Q_OS_WIN) && defined(__MINGW32__))

// When compiled with MinGW, only allow MinGW as module compiler
const QString defCompiler = MingwCC;
const QStringList compilerList = (QStringList() << MingwCC);
const QString defTmpPath = "C:/Windows/Temp";

#define isnan	std::isnan

#else

const QString defCompiler = clangCC;
const QStringList compilerList = (QStringList() << clangCC << gnuCC << intelCC);
const QString defTmpPath = "/tmp";

#endif

#if !defined(TRUE)
#define TRUE	true
#endif

#if !defined(FALSE)
#define FALSE	false
#endif

const QString WINDOWS_REGISTRY = "/QtApplications";
const QString APP_KEY = "/EBTtool/";

const int OUTPUTVALUES = 0x01;
const int COMPUTEDVALUES = 0x02;
const int FIXEDVALUES = 0x04;
const int STATEOUTPUT = 0x08;
const int XAXIS = 0x01;
const int YAXIS = 0x02;

const uint APPENDDATA = 1;
const uint REPLACEDATA = 0;

const uint REDRAWPLOT = 2;
const uint REDRAWCURVES = 1;
const uint DRAWPOINTS = 0;

const int defaultGIFDelay = 10;

// the qwt libs don't prevent plotting data <= 0 on a logarithmic scale which would cause a crash
// when the tool is compiled with certain compilers; the value below is ok for gcc and bcc32
// const double MIN_LOG_ARG = 1.17549435E-38F;
const double MIN_LOG_VAL = 1.0E-15;
const double MAX_LOG_VAL = 1.0E+100;
const double ZERO_VAL	 = 1.0E-16;

// Maximum number of tick marks with autoscaling
const int	      MAX_AUTO_TICKS = 8;

// Default number of tick marks on logarithmic scales
const int	      MINOR_LOG_TICKS = 9;

QString formatDouble(double);
double myLog10(double);

#endif //EBTDEFINES_H
