#include <QFileDialog>
#include <qmessagebox.h>
#include <qfileinfo.h>
#include <QSettings>
#include <QMessageBox>
#include "ebtfile.h"
#include <QtDebug>

EBTFile::EBTFile()
{
    setFileName(QString());
    setDirPath(QString());
    setChanged(FALSE);
    setNew(FALSE);
    setEditor(0);
}

EBTFile::~EBTFile()
{
}

void EBTFile::setFileName(const QString& str)
{
    if(str.isEmpty())
	filename = QString();
    else
    {
	QFileInfo fi(str);
	filename = fi.fileName();
	setDirPath(fi.absolutePath());
    }
}


void EBTFile::resetFileName()
{
    // this function is used to reset all the file's children files to the basename
    // which is useful if a child file has been saved with another name (save as...) 
    setFileName(absFilePath());
}


void EBTFile::createNew()
{
    setFileName(QString());
    setNew(TRUE);
}

QString EBTFile::captionText()
{
    QString caption = EBTCAPTION;
    if(!fileName().isEmpty())
	caption.append(fileName());
    else
	caption.append("[New " + descriptionText() + "]");
    
    return caption;
}

bool EBTFile::open()
{
    QString fName = QFileDialog::getOpenFileName(NULL, "EBTtool -- file open", dirPath(), openText());
    if (!fName.isEmpty())
    {
	setFileName(fName);
	return TRUE;
    }
    else
	return FALSE;
    //Specifieke code voor betreffende bestandstype in de afgeleide klasse
}

void EBTFile::load()
{
    // nothing here
}

bool EBTFile::saveAs()
{
    QString fName = QFileDialog::getSaveFileName(NULL, "EBTtool -- file save as", QString(), saveText());
    if (!fName.isEmpty())
    {
	QFileInfo fi(fName);
	if(!fileExtension().isEmpty())
	{
	    if (fi.suffix() != fileExtension())
		fName.append(QString("." + fileExtension()));
	}
	
	setFileName(fName);
	bool ok = save();
	if (!ok)
	  {
	    setFileName(QString());
	    return FALSE;
	  }
	else
	  setNew(FALSE);
	return TRUE;
    }   
    return FALSE;
}

QString EBTFile::copyAs(const QString & newName)
{
  QString fName;

  if (newName.isNull() || newName.isEmpty())
    {
      fName = QFileDialog::getSaveFileName(NULL, "EBTtool -- file save as", QString(), saveText());
      if (fName.isEmpty()) return QString();
    }
  else
    {
      if (QFile(newName).exists())
	{
	  if(QMessageBox::question( NULL, EBTCAPTION + "Overwrite file?",
				    QString("File %1 already exists.\n").arg(newName) + 
				    QString("Do you want to replace it?"),
				    QMessageBox::Yes, QMessageBox::No)
	     == QMessageBox::No) return QString();
	}
      fName = newName;
    }

  QFileInfo fi(fName);
  if(!fileExtension().isEmpty())
    {
      if (fi.suffix() != fileExtension())
	fName.append(QString("." + fileExtension()));
    }
  if (QFile(fName).exists()) QFile(fName).remove();
  bool ok = QFile(absFilePath()).copy(fName);

  if (!ok) return QString();
  else return fName;
}

bool EBTFile::okToClear()
{
  if (isChanged())
    {
      QString msg;
      if (fileName().isEmpty())
        msg = "Unnamed " + descriptionText() + "\n";
      else
        msg = QString("File '%1'\n").arg(fileName());
      msg += QString("has been changed.");
      QMessageBox::StandardButton ans = QMessageBox::question(0, QString(EBTCAPTION + "Unsaved changes"), msg, QMessageBox::Save | QMessageBox::Cancel | QMessageBox::Discard, QMessageBox::Save);
      if (ans == QMessageBox::Save)
        {
          if (fileName().isEmpty()) saveAs();
          else save();                                                                  // ok: calls save() routine of subclass
        }
      else if (ans == QMessageBox::Cancel) return FALSE;
    }
  return TRUE;
}

void EBTFile::loadSettings()
{
    QSettings settings;

    EBTDir = settings.value("EBTDir").toString();
}
