#ifndef EBTFILE_H
#define EBTFILE_H

#include <qstring.h>
#include <qdir.h>
#include "ebtdefines.h"

class TextEditor;

class EBTFile {
protected:
    QString EBTDir;

private:
    EBTFile(const EBTFile&);   // declare a private copy constructor, to prevent passing by value
    QString templatefile;
    QString fileextension;    
    QString descriptiontext;
    QString opentext;
    QString savetext;
    QString filename;
    QString dirpath;
    bool ischanged;
    bool isnew;
    TextEditor *editor;
    
public:
    EBTFile();
    virtual ~EBTFile();
    void setTemplateFile(const QString& str){templatefile = str;};
    void setFileExtension(const QString& str){fileextension = str;};
    void setDescriptionText(const QString& str){descriptiontext = str;};
    void setOpenText(const QString& str){opentext = str;};
    void setSaveText(const QString& str){savetext = str;};
    void setDirPath(const QString& str){dirpath = str;};
    void setChanged(const bool bl){ischanged = bl;};
    void setNew(const bool bl){isnew = bl;};
    void setEditor(TextEditor *ed){editor = ed;};
    
    QString templateFile() {return templatefile;};
    QString fileExtension() {return fileextension;};    
    QString descriptionText() {return descriptiontext;};
    QString openText() {return opentext;};
    QString saveText() {return savetext;};
    QString fileName() {return filename;};
    QString dirPath() {return dirpath;};
    QString absFilePath() {return dirpath + "/" + filename;};
    bool isChanged() {return ischanged;};
    bool isNew() {return isnew;};
    QString captionText();
    TextEditor *inEditor() {return editor;};
    
    virtual void setFileName(const QString& str);
    void resetFileName();
    void loadSettings();
    bool saveAs();
    QString copyAs(const QString & newName);
    bool okToClear();
    void createNew();
    virtual bool open();
    virtual void load();
    virtual bool save() { return FALSE;};		// save() is implemented in derived classes
    
    QString buffer;
};

#endif // EBTFILE_H
