#include <qmessagebox.h>
#include "ebtdefines.h"
#include "ebtgifmovie.h"

unsigned long masks[] = { 0x0000, 0x0001, 0x0003, 0x0007, 0x000F, 0x001F, 
			  0x003F, 0x007F, 0x00FF, 0x01FF, 0x03FF,  0x07FF, 
			  0x0FFF, 0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF };

EBTGIF::EBTGIF(QString fName)
{
    outputFile = fopen(fName.toLatin1().constData (), "wb");				// Qt4
    Pass = free_ent = clear_flg = out_count = 0;
    maxbits = GIFBits;
    maxmaxcode = (int) 1 << GIFBits;
    in_count = 1;
}

EBTGIF::~EBTGIF()
{
}

void EBTGIF::initializeGIF(uint fTime, QImage image)
{
    int empty[colorTableSize];
    frameTime = fTime;
    
    /* Start with an empty color table; this part of the file will be overwritten at file close */
    colorMap.clear();
    for(uint ii = 0; ii < colorTableSize; ii++)
	empty[ii] = 0;
    
    filePosColorTable = 0L;
    filePosEnd = 0L;

    /* Now save an image in a compound GIF */
    GIFEncodeHeader(1, image.width(), image.height(), 0 /*interlace*/, 0, image.depth(),
		    empty, empty, empty);
    
    /* Animated GIF with infinite-looping extension */
    GIFEncodeLoopExt(0);
    
    /* Write comment extension - mustn't be written before the looping ext. */
    char comment[] = "EBTtool GIF file";
    GIFEncodeCommentExt(comment);
    
    writeGIFImage(image);
}

void EBTGIF::writeGIFImage(QImage image)
{
    currentImage = image;
    
    GIFEncodeGraphicControlExt( 2, frameTime, 2, currentImage.width(), currentImage.height(), 
			       0 /*interlace*/, 0 /*, image.depth()*/);

    GIFEncodeImageData(currentImage.width(), currentImage.height(), 0 /*interlace*/, 
		       currentImage.depth(), /*getPixel, */ 0 /* left offset */, 0 /* top offset*/);
}

int EBTGIF::getPixel(int x, int y)
{
    uint ii;
    
    QRgb pixelValue = currentImage.pixel(x, y);
    
    bool colorInTable = FALSE;
    
    uint colorMapSize = colorMap.size();
    for(ii = 0; ii < colorMapSize; ii++)
    {
	if(pixelValue == colorMap[ii])
	{
	    colorInTable = TRUE;
	    break;
	}
    }
    
    if ((!colorInTable) && (!colorMap.contains(colorMapSize)))
	colorMap.insert(colorMapSize, pixelValue);
    
    return (int)ii;
}

void EBTGIF::finishGIF()
{
    filePosEnd = ftell(outputFile);
    fseek(outputFile, filePosColorTable, SEEK_SET);
    // write header anew, with correct color table
    for(uint ii = 0; ii < colorTableSize; ++ii)
    {
	if(colorMap.contains(ii))
	{
	    fputc(qRed(colorMap[ii]), outputFile);
	    fputc(qGreen(colorMap[ii]), outputFile);
	    fputc(qBlue(colorMap[ii]), outputFile);
	}
	else
	    for(uint jj = 0; jj < 3; ++jj)
		fputc(0, outputFile);
    }
    fseek(outputFile, filePosEnd, SEEK_SET);
    GIFEncodeClose();
}
    
void EBTGIF::GIFEncodeHeader(int gif89, int GWidth, int GHeight, int GInterlace, int Background, 
		     int BitsPerPixel, int Red[], int Green[], int Blue[])
{
    Interlace = GInterlace;
    
    int ColorMapSize = 1 << BitsPerPixel;
    int Resolution = BitsPerPixel;
    
    Width = GWidth;
    Height = GHeight;
    
    // Calculate number of bits we are expecting
    CountDown = (long) Width *(long) Height;
    
    // Indicate which pass we are on (if interlace)
    Pass = 0;
    
    // Set up the current x and y position
    curx = cury = 0;
    
    // Write the Magic header
    (void)fwrite (gif89 ? "GIF89a" : "GIF87a", 1, 6, outputFile);
    
    // Write out the screen width and height
    Putword (GWidth);
    Putword (GHeight);
    
    // Indicate that there is a global colour map
    int B = 0x80;	// Yes, there is a color map
    
    // OR in the resolution
    B |= (Resolution - 1) << 5;
    
    // OR in the Bits per Pixel
    B |= (BitsPerPixel - 1);
    
    // Write it out
    fputc (B, outputFile);
    
    // Write out the Background colour
    fputc (Background, outputFile);
    
    // Byte of 0's (future expansion)
    fputc (0, outputFile);
    
    filePosColorTable = ftell(outputFile);
    
    // Write out the Global Colour Map
    for(int ii = 0; ii < ColorMapSize; ++ii)
    {
	fputc (Red[ii], outputFile);
	fputc (Green[ii], outputFile);
	fputc (Blue[ii], outputFile);
    }
}


void EBTGIF::GIFEncodeGraphicControlExt(int Disposal, int Delay89, int NumFramesInImage, 
				int GWidth, int GHeight, int GInterlace, int Transparent /*, int BitsPerPixel */)
{
    Interlace = GInterlace;
    
    Width = GWidth;
    Height = GHeight;
    
    // Calculate number of bits we are expecting
    CountDown = (long) Width *(long) Height;
    
    // Indicate which pass we are on (if interlace)
    Pass = 0;
    
    // Set up the current x and y position
    curx = cury = 0;
    
    // Write out extension for transparent colour index, if necessary.
    if ( (Transparent >= 0) || (NumFramesInImage > 1) )
    {
	// Extension Introducer - fixed.
	fputc ('!', outputFile);
	// Graphic Control Label - fixed.
	fputc (0xf9, outputFile);
	// Block Size - fixed.
	fputc (4, outputFile);
	
	// Packed Fields - XXXdddut (d=disposal, u=userInput, t=transFlag)
	//                    s8421
	fputc (((Transparent >= 0) ? 0x01 : 0x00)				// TRANSPARENCY
	       | ((NumFramesInImage > 1) ? (Disposal << 2) : 0x00 ),	// DISPOSAL
	       // 0x03 or 0x01 build frames cumulatively
	       // 0x02 clears frame before drawing
	       // 0x00 'don't care'
	       outputFile);
	
	fputc (Delay89 & 255, outputFile);
	fputc ((Delay89>>8) & 255, outputFile);
	
	fputc (Transparent, outputFile);
	fputc (0, outputFile);
    }
}


void EBTGIF::GIFEncodeImageData(int GWidth, int GHeight, int GInterlace, int BitsPerPixel,
			/*ifunptr  GetPixel, */ int offset_x, int offset_y)
{
    int LeftOfs, TopOfs, InitCodeSize;
    
    Interlace = GInterlace;
    
    Width = GWidth;
    Height = GHeight;
    LeftOfs = (int) offset_x;
    TopOfs = (int) offset_y;
    
    // Calculate number of bits we are expecting
    CountDown = (long) Width *(long) Height;
    
    // Indicate which pass we are on (if interlace)
    Pass = 0;
    
    // The initial code size
    if (BitsPerPixel <= 1)
	InitCodeSize = 2;
    else
	InitCodeSize = BitsPerPixel;
    
    // Set up the current x and y position
    curx = cury = 0;
    
    // Write an Image separator
    fputc (',', outputFile);
    
    // Write the Image header
    Putword (LeftOfs);
    Putword (TopOfs);
    Putword (Width);
    Putword (Height);
    
    // Write out whether or not the image is interlaced
    if (Interlace)
	fputc (0x40, outputFile);
    else
	fputc (0x00, outputFile);
    
    // Write out the initial code size
    fputc (InitCodeSize, outputFile);
    
    // Go and actually compress the data
    compress (InitCodeSize + 1 /*, GetPixel*/);
    
    // Write out a Zero-length packet (to end the series)
    fputc (0, outputFile);
}


void EBTGIF::GIFEncodeClose()
{
    // Write the GIF file terminator
    fputc (';', outputFile);
    return;
}


void EBTGIF::GIFEncodeLoopExt(uint num_loops)
{
    fputc(0x21,outputFile);
    fputc(0xff,outputFile);
    fputc(0x0b,outputFile);
    fputs("NETSCAPE2.0",outputFile);
    fputc(0x03,outputFile);
    fputc(0x01,outputFile);
    Putword(num_loops);
    fputc(0x00,outputFile);
    
    // NOTE: num_loops==0 means 'loop infinitely'
}


void EBTGIF::GIFEncodeCommentExt(char *comment)
{
    if (comment==NULL||strlen(comment)<1) return;
    
    fputc(0x21,outputFile);
    fputc(0xfe,outputFile);
    fputc(strlen(comment),outputFile);
    fputs((const char *)comment,outputFile);
    fputc(0x00,outputFile);
}


/*
 * GIF Image compression - modified 'compress'
 *
 * Based on: compress.c - File compression ala IEEE Computer, June 1984.
 *
 * By Authors:  Spencer W. Thomas
 *              Jim McKie
 *              Steve Davies
 *              Ken Turkowski
 *              James A. Woods
 *              Joe Orost
 */

/*
 * To save much memory, we overlay the table used by compress() with those
 * used by decompress().  The tab_prefix table is the same size and type
 * as the codetab.  The tab_suffix table needs 2**GIF_BITS characters.  We
 * get this from the beginning of htab.  The output stack uses the rest
 * of htab, and contains characters.  There is plenty of room for any
 * possible stack (stack used to be 8000 characters).
 */

/*
 * compress stdin to stdout
 *
 * Algorithm:  use open addressing double hashing (no chaining) on the
 * prefix code / next character combination.  We do a variant of Knuth's
 * algorithm D (vol. 3, sec. 6.4) along with G. Knott's relatively-prime
 * secondary probe.  Here, the modular division first probe is gives way
 * to a faster exclusive-or manipulation.  Also do block compression with
 * an adaptive reset, whereby the code table is cleared when the compression
 * ratio decreases, but after the table fills.  The variable-length output
 * codes are re-sized at this point, and a special CLEAR code is generated
 * for the decompressor.  Late addition:  construct the table according to
 * file size for noticeable speed improvement on small files.  Please direct
 * questions about this implementation to ames!jaw.
 */

void EBTGIF::compress(int init_bits /*, ifunptr ReadValue*/)
{
    long		fcode;
    int		i, c, ent, disp, hsize_reg, hshift;
    
    
    /*
   * Set up the globals:  g_init_bits - initial number of bits
   *                      g_outfile   - pointer to output file
   */
    g_init_bits = init_bits;
    g_outfile = outputFile;	// waarom nieuwe pointer?
    
    cur_bits = 0;
    cur_accum = 0;
    
    // Set up the necessary values
    offset = 0;
    out_count = 0;
    clear_flg = 0;
    in_count = 1;
    
    ClearCode = (1 << (init_bits - 1));
    EOFCode = ClearCode + 1;
    free_ent = ClearCode + 2;
    
    
    // Had some problems here... should be okay now.  --Adam
    n_bits = g_init_bits;
    maxcode = MAXCODE (n_bits);
    
    a_count = 0;
    
    ent = GIFNextPixel(/*ReadValue*/);
    
    hshift = 0;
    for (fcode = (long) hsize; fcode < 65536L; fcode *= 2L)
	++hshift;
    hshift = 8 - hshift;				// set hash code range bound
    
    hsize_reg = hsize;
    cl_hash ((long int) hsize_reg);		// clear hash table
    
    output ((int) ClearCode);
    
    
    while ((c = GIFNextPixel (/*ReadValue*/)) != EOF)
    {				/* } */
	
	++in_count;
	
	fcode = (long) (((long) c << maxbits) + ent);
	i = (((int) c << hshift) ^ ent);	/* xor hashing */
	
	if (HashTabOf (i) == fcode)
	{
	    ent = CodeTabOf (i);
	    continue;
	}
	else if ((long) HashTabOf (i) < 0)	// empty slot
	    goto nomatch;
	disp = hsize_reg - i;			// secondary hash (after G. Knott)
	if (i == 0)
	    disp = 1;
	probe:
	    if ((i -= disp) < 0)
		i += hsize_reg;
	    
	    if (HashTabOf (i) == fcode)
	    {
		ent = CodeTabOf (i);
		continue;
	    }
	    if ((long) HashTabOf (i) > 0)
		goto probe;
	    nomatch:
		output ((int) ent);
		++out_count;
		ent = c;
		if (free_ent < maxmaxcode)
		{			/* } */
		    CodeTabOf (i) = free_ent++;		// code -> hashtable
		    HashTabOf (i) = fcode;
		}
		else
		    cl_block ();
	    }
    
    // Put out the final code.
    output ((int) ent);
    ++out_count;
    output ((int) EOFCode);
}

/*****************************************************************
 * TAG( output )
 *
 * Output the given code.
 * Inputs:
 *      code:   A n_bits-bit integer.  If == -1, then EOF.  This assumes
 *              that n_bits =< (long)wordsize - 1.
 * Outputs:
 *      Outputs code to the file.
 * Assumptions:
 *      Chars are 8 bits long.
 * Algorithm:
 *      Maintain a GIF_BITS character long buffer (so that 8 codes will
 * fit in it exactly).  Use the VAX insv instruction to insert each
 * code in turn.  When the buffer fills up empty it and start over.
 */

void EBTGIF::output(int code)
{
    cur_accum &= masks[cur_bits];
    
    if (cur_bits > 0)
	cur_accum |= ((long) code << cur_bits);
    else
	cur_accum = code;
    
    cur_bits += n_bits;
    
    while (cur_bits >= 8)
    {
	char_out ((uint) (cur_accum & 0xff));
	cur_accum >>= 8;
	cur_bits -= 8;
    }
    
    /*
   * If the next entry is going to be too big for the code size,
   * then increase it, if possible.
   */
    if (free_ent > maxcode || clear_flg)
    {
	if (clear_flg)
	{
	    maxcode = MAXCODE (n_bits = g_init_bits);
	    clear_flg = 0;
	}
	else
	{
	    ++n_bits;
	    if (n_bits == maxbits)
		maxcode = maxmaxcode;
	    else
		maxcode = MAXCODE (n_bits);
	}
    }
    
    if (code == EOFCode)
    {
	// At EOF, write the rest of the buffer.
	while (cur_bits > 0)
	{
	    char_out ((uint) (cur_accum & 0xff));
	    cur_accum >>= 8;
	    cur_bits -= 8;
	}
	
	flush_char ();
	fflush (g_outfile);
	
	if (ferror(g_outfile)) 
	    QMessageBox::critical(0, EBTCAPTION + "error writing file", "GIF: error writing output file");
    }
}

// Clear out the hash table

void EBTGIF::cl_block()				// table clear for block compress
{
    cl_hash ((long int) hsize);
    free_ent = ClearCode + 2;
    clear_flg = 1;
    output ((int) ClearCode);
}

void EBTGIF::cl_hash(long int hsize)	// reset code table
{
    long int	*htab_p = htab + hsize;
    long	i, m1 = -1;
    
    i = hsize - 16;
    do
    {						// might use Sys V memset(3) here
	*(htab_p - 16) = m1;
	*(htab_p - 15) = m1;
	*(htab_p - 14) = m1;
	*(htab_p - 13) = m1;
	*(htab_p - 12) = m1;
	*(htab_p - 11) = m1;
	*(htab_p - 10) = m1;
	*(htab_p - 9) = m1;
	*(htab_p - 8) = m1;
	*(htab_p - 7) = m1;
	*(htab_p - 6) = m1;
	*(htab_p - 5) = m1;
	*(htab_p - 4) = m1;
	*(htab_p - 3) = m1;
	*(htab_p - 2) = m1;
	*(htab_p - 1) = m1;
	htab_p -= 16;
    }
    while ((i -= 16) >= 0);
    
    for (i += 16; i > 0; --i) *--htab_p = m1;
}

/******************************************************************************
 *
 * GIF Specific routines
 *
 ******************************************************************************/

// Bump the 'curx' and 'cury' to point to the next pixel

void EBTGIF::BumpPixel ()
{
    // Bump the current X position
    ++curx;
    
    /*
   * If we are at the end of a scan line, set curx back to the beginning
   * If we are interlaced, bump the cury to the appropriate spot,
   * otherwise, just increment it.
   */
    if (curx == Width)
    {
	curx = 0;
	if (!Interlace) ++cury;
	else
	{
	    switch (Pass)
	    {
		    case 0:
			cury += 8;
			if (cury >= Height)
			{
			    ++Pass;
			    cury = 4;
			}
			break;
		    case 1:
			cury += 8;
			if (cury >= Height)
			{
			    ++Pass;
			    cury = 2;
			}
			break;
		    case 2:
			cury += 4;
			if (cury >= Height)
			{
			    ++Pass;
			    cury = 1;
			}
			break;
		    case 3:
			cury += 2;
			break;
		    }
	}
    }
}


// Return the next pixel from the image

int EBTGIF::GIFNextPixel (/*ifunptr getpixel*/)
{
    int r;
    
    if (CountDown == 0) return EOF;
    
    --CountDown;
    
//    r = (*getpixel) (curx, cury);
    r = getPixel(curx, cury);
    
    BumpPixel();
    
    return r;
}

// Write out a word to the GIF file
void EBTGIF::Putword (int w)
{
    fputc (w & 0xff, outputFile);
    fputc ((w / 256) & 0xff, outputFile);
}

// Add a character to the end of the current packet, and if it is 254
// characters, flush the packet to disk.
void EBTGIF::char_out (int c)
{
    accum[a_count++] = c;
    if (a_count >= 254) flush_char ();
}

// Flush the packet to disk, and reset the accumulator
void EBTGIF::flush_char ()
{
    if (a_count > 0)
    {
	fputc (a_count, g_outfile);
	fwrite (accum, 1, a_count, g_outfile);
	a_count = 0;
    }
    return;
}
