/***
   NAME
     ebtgifmovie
   DESCRIPTION
     Module to write an animated GIF89a file as output.
     
     This filter uses code taken from the GIF loading and saving file filter
     for The GIMP version 1.0 (Version 2.0.5 - 98/09/28), based on code taken
     from the "giftopnm" and "ppmtogif" programs which are part of the "netpbm"
     package. 
     
     ** ppmtogif.c - read a portable pixmap and produce a GIF file
     **
     ** Based on GIFENCOD by David Rowley <mgardi@watdscu.waterloo.edu>.A
     ** Lempel-Zim compression based on "compress".
     **
     ** Modified by Marcel Wijkstra <wijkstra@fwi.uva.nl>
     **
     ** Copyright (C) 1989 by Jef Poskanzer.
     **
     ** Permission to use, copy, modify, and distribute this software and its
     ** documentation for any purpose and without fee is hereby granted, provided
     ** that the above copyright notice appear in all copies and that both that
     ** copyright notice and this permission notice appear in supporting
     ** documentation.  This software is provided "as is" without express or
     ** implied warranty.
     **
     ** The Graphics Interchange Format(c) is the Copyright property of
     ** CompuServe Incorporated.  GIF(sm) is a Service Mark property of
     ** CompuServe Incorporated.
     
   Last modification: AMdR - Jan 23, 2006
***/
#ifndef EBTGIFMOVIE_H
#define EBTGIFMOVIE_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifndef Q_OS_WIN
// is sys/param.h necessary anyway?
#include <sys/param.h>
#endif
#include <qstring.h>
#include <qimage.h>
#include <QMap>

// Pointer to function returning an int
//typedef int (*ifunptr) (int, int);
#ifdef NO_UCHAR
typedef char char_type;
#else
typedef unsigned char char_type;
#endif
#ifdef COMPATIBLE			// But wrong!
#define MAXCODE(Mn_bits)	((int) 1 << (Mn_bits) - 1)
#else
#define MAXCODE(Mn_bits)(((int) 1 << (Mn_bits)) - 1)
#endif
const uint colorTableSize = 256;
const int GIFBits = 12;
const int hsize = 5003;			// 80% occupancy
#define HashTabOf(i)       htab[i]
#define CodeTabOf(i)    codetab[i]
#define tab_prefixof(i) CodeTabOf(i)
#define tab_suffixof(i) ((char_type*)(htab))[i]
//#define de_stack ((char_type*)&tab_suffixof((int)1<<GIFBits))
//#define ARGVAL()	(*++(*argv) || (--argc && *++argv))

class EBTGIF
{
public:
    EBTGIF(QString fName);
    ~EBTGIF();
    void initializeGIF(uint frameTime, QImage image);
    void writeGIFImage(QImage image);
    int getPixel(int x, int y);
    void finishGIF();
    
private:
    void GIFEncodeHeader(int, int, int, int, int, int, int *, int *, int *);
    void GIFEncodeGraphicControlExt(int, int, int, int, int, int, int /*, int*/);
    void GIFEncodeImageData(int, int, int, int, /*ifunptr,*/ int, int);
    void GIFEncodeClose();
    void GIFEncodeLoopExt(uint);
    void GIFEncodeCommentExt(char *);
    void BumpPixel(void);
    int GIFNextPixel(/*ifunptr*/);
    void Putword (int);
    void compress (int /*, ifunptr*/);
    void output (int);
    void cl_block (void);
    void cl_hash (long int);
    void char_out (int);
    void flush_char (void);
    
    FILE *outputFile;	//ipv QFile!
    QImage currentImage;
    QMap<uint, QRgb> colorMap;
    //    QRgb colors[256];
    long int filePosColorTable;
    long int filePosEnd;
    
    uint frameTime;
    int Width, Height;
    int curx, cury;
    long CountDown;
    int Pass;
    int Interlace;
    
    int n_bits;			// number of bits/code
    int maxbits;		// user settable max # bits/code
    int maxcode;		// maximum code, given n_bits
    int maxmaxcode;		 // should NEVER generate this code
    
    long int htab[hsize];
    unsigned short codetab[hsize];
    int free_ent;		// first unused entry
    
    /*
 * block compression parameters -- after all codes are used up,
 * and compression rate changes, start over.
 */
    int clear_flg;
    int offset;
    long int in_count;		// length of input
    long int out_count;		// # of codes output (for debugging)
    
    int g_init_bits;
    FILE *g_outfile;
    
    int ClearCode;
    int EOFCode;
    
    unsigned long cur_accum;
    int cur_bits;
    // Number of characters so far in this 'packet'
    int a_count;
    // Define the storage for the packet accumulator
    char accum[256];
};    

#endif	//EBTGIFMOVIE_H
