/***
   NAME
     ebtgraph - header file
   DESCRIPTION
     
   Last modification: AMdR - Nov 13, 2014
***/
#ifndef EBTGRAPH
#define EBTGRAPH
#include <QMainWindow>

#include "ui_ebtgraph.h"   // defines Ui::EBTGraphForm
#include <qwt_plot_directpainter.h>
#include "qwt_plot_grid.h"

class LegendItem;

class EBTGraphForm : public QMainWindow, public Ui::EBTGraphForm
{
    Q_OBJECT

public:
    EBTGraphForm(QWidget* parent = 0, Qt::WindowFlags fl = Qt::Window);
    ~EBTGraphForm();

    EBTGraphSettings currentSettings;

    virtual void incrementalOutputPlot();
    virtual void setFile( EBTGrFile & ef );
    virtual EBTGrFile * getFile() { return currentFile; };
    virtual void applySettings();
    virtual bool parentClose();
    virtual QString absFilePath();
    virtual bool hasAutoScale();
    virtual bool hasAutoScaleX( uint setNr );
    virtual double axisMin( QwtPlot::Axis axis );
    virtual double axisMax( QwtPlot::Axis axis );
    virtual double axisMinX( uint setNr );
    virtual double axisMaxX( uint setNr );
    virtual double axisMinY( uint setNr );
    virtual double axisMaxY( uint setNr );
    virtual QString fileName();
    virtual void showText( const QString & text );
    virtual QwtPlot * getPlot() { return thisPlot; };
    virtual QAction * getGraphMenuItem() { return graphMenuItem; };
    virtual void setGraphMenuItem(QAction * action) { graphMenuItem = action; return ; };
    virtual void setHideFlag( bool fl ) { hiddenFlag = fl; };
    virtual bool getHideFlag() { return hiddenFlag; };
    virtual void setDirtyFlag( bool fl ) { dirtyFlag = fl; };
    virtual bool getDirtyFlag() { return dirtyFlag; };

public slots:
    virtual void fileNew();
    virtual void fileOpen();
    virtual void fileSaveAs();
    virtual void fileSave();
    virtual void filePrint();
    virtual void closeEvent( QCloseEvent * );
    virtual void fileExit();
    virtual void helpIndex();
    virtual void helpContents();
    virtual void helpAbout();
    virtual void layoutEdit();
    virtual void cleanUp();
    virtual void redrawData();
    virtual void setSettings( EBTGraphSettings newsettings );
    virtual void newGraph();
    virtual void axisToLeft();
    virtual void axisToRight();
    virtual void plotSavePNG();
    virtual void updateMenu( bool disable );
    virtual void deleteFile( QString fName );
    virtual void plotSaveMovie();
    virtual void saveMovieFrame();
    virtual void plotFinishMovie();
    virtual void plotCleanupMovie( bool discard );
    virtual bool makeHistogram( uint setNr );
    virtual void graphShowAndRaiseSlot();

signals:
    void newGrFile(EBTGraphForm *, bool);
    void openNewGraph(bool);
    void closing(EBTGraphForm *);
    void minimizing(EBTGraphForm *);
    void offScale(EBTGraphForm *, bool);

protected slots:
    virtual void languageChange();

protected:
    void hideEvent ( QHideEvent * );
    void showEvent ( QShowEvent * );

private:
    EBTGIF *movie;
    QString movieName;
    bool recordingMovie;
    uint currentGIFDelay;
    bool autoScale;
    QwtPlotZoomer *zoomer;
    bool isOutputGraph;
    EBTGrFile *currentFile;
    QwtPlot *thisPlot;
    QwtPlotDirectPainter *plotDirectPainter;
    LegendItem *floatingLegend;
    QwtPlotGrid *plotGrid;
    QString currentText;
    bool zoomOn;
    double histYMax;
    QAction *graphMenuItem;
    bool hiddenFlag;
    bool dirtyFlag;

    void init();
    virtual bool okToClear();

private slots:
    virtual void zoomEnable( bool on );

};

#endif /* EBTGRAPH */
