#include <qfile.h>
#include "ebtgraphsettings.h"

EBTGraphSettings::EBTGraphSettings()
{
    clearDataSetMap();
    
    axis xBottom, xTop, yLeft, yRight;
    xBottom.autoscale = xTop.autoscale = yLeft.autoscale = yRight.autoscale = FALSE;
    xBottom.logarithmic = xTop.logarithmic = yLeft.logarithmic = yRight.logarithmic = FALSE;
    xBottom.minVal = xTop.minVal = yLeft.minVal = yRight.minVal = 0.0;
    xBottom.maxVal = xTop.maxVal = yLeft.maxVal = yRight.maxVal = 1000.0;
    axisMap[QwtPlot::xBottom] = xBottom; 
    axisMap[QwtPlot::xTop] = xTop;
    axisMap[QwtPlot::yLeft] = yLeft;
    axisMap[QwtPlot::yRight] = yRight;
 
    xsize = ysize = 15;
    outgraph = defaultdescription = TRUE;
    showlegend = showtimestamp = showdescription = legendframed = FALSE;
    legendframewidth = 1;
    legendposition = QwtPlot::BottomLegend;
    legendxposition = legendyposition = 1.0;
    showgrid = showfinegrid = FALSE;
    allsetssameas = -1;
}

EBTGraphSettings::~EBTGraphSettings()
{
}


uint EBTGraphSettings::addDataSet()
{
    uint currentSetKey = dataSetMap.size();
    EBTDataSet *aset = new EBTDataSet;
    if (!dataSetMap.contains(currentSetKey))
      dataSetMap.insert(currentSetKey, *aset);
    delete aset;
    
    return currentSetKey;
}

void EBTGraphSettings::removeDataSet(uint setNo)
{
    if(!dataSetMap.contains(setNo))
	return;
    
    dataSetMap.remove(setNo);
    
    // now make sure the map containing the datasets is indexed correctly again:
    EBTDataSetMap copyMap(dataSetMap);    
    clearDataSetMap();
    EBTDataSetMap::Iterator i;
    for(i = copyMap.begin(); i != copyMap.end(); ++i)
    {	
	uint currentKey = dataSetMap.size();
	dataSetMap.insert(currentKey, i.value());
    }
}

void EBTGraphSettings::clearDataSet(uint setNr)
{
    dataSetMap[setNr].clear();
}

void EBTGraphSettings::clearDataSetMap()
{
    for(int ii = 0; ii < dataSetMap.size(); ii++)
	clearDataSet(ii);
    dataSetMap.clear();
}
