#ifndef EBTGRSETTINGS_H
#define EBTGRSETTINGS_H

#include "ebtdataset.h"

class EBTGraphSettings
{
private:
    struct axis
    {
	bool logarithmic;
	QString title;
	QFont titleFont;
	bool autoscale;
	double minVal;
	double maxVal;
    };
    QMap< QwtPlot::Axis /*yLeft,  yRight,  xBottom,  xTop, axisCnt*/, axis> axisMap;
    
    bool outgraph;
    int xpos;
    int xsize;
    int ypos;
    int ysize;
    QString plottitle;
    QFont plottitlefont;
    bool showgrid;
    bool showfinegrid;
    bool showlegend;
    bool legendframed;
    int  legendframewidth;
    int legendposition;
    double legendxposition;
    double legendyposition;
    bool showtimestamp;
    bool showdescription;
    bool defaultdescription;
    QString plotdescription;
    QFont legendfont;
    
    EBTDataSetMap dataSetMap;
    int allsetssameas;
    
public:    
    EBTGraphSettings();
    ~EBTGraphSettings();
    
    uint addDataSet();
    void removeDataSet(uint setNr);
    void clearDataSet(uint setNr);
    void clearDataSetMap();
    
    int xPos(){ return xpos; };
    int xSize(){ return xsize; };
    int yPos(){ return ypos; };
    int ySize(){ return ysize; };
    QString plotTitle(){ return plottitle; };
    QFont plotTitleFont(){ return plottitlefont; };
    bool showGrid(){ return showgrid; };
    bool showFineGrid(){ return showfinegrid; };
    bool showLegend(){ return showlegend; };
    bool legendFramed() { return legendframed; };
    int  legendFrameWidth() { return legendframewidth; };
    int  legendPosition() { return legendposition; };
    double legendXPosition() { return legendxposition; };
    double legendYPosition() { return legendyposition; };
    bool showTimestamp(){ return showtimestamp; };
    bool showDescription(){ return showdescription; };
    bool defaultDescription(){ return defaultdescription; };
    QString plotDescription(){ return plotdescription; };
    QFont legendFont(){ return legendfont; };
    int allSetsSameAs(){ return allsetssameas; };
    
    bool axisExists(QwtPlot::Axis a){ return axisMap.contains(a); };
    bool axisLogarithmic(QwtPlot::Axis a){ return axisMap[a].logarithmic; };
    QString axisTitle(QwtPlot::Axis a){ return axisMap[a].title; };
    QFont axisTitleFont(QwtPlot::Axis a){ return axisMap[a].titleFont; };
    bool axisAutoscale(QwtPlot::Axis a){ return axisMap[a].autoscale; };
    double axisMinVal(QwtPlot::Axis a){ return axisMap[a].minVal; };
    double axisMaxVal(QwtPlot::Axis a){ return axisMap[a].maxVal; };
    
    bool dataSetExists(uint setNr){ return dataSetMap.contains(setNr); };
    uint dataSetMapSize(){ return dataSetMap.size(); };
    uint dataSetXCol(uint setNr){ return dataSetMap[setNr].xCol(); };
    uint dataSetYCol(uint setNr){ return dataSetMap[setNr].yCol(); };
    double * dataSetXDataPnt(uint setNr){ return dataSetMap[setNr].xDataPnt(); };
    double * dataSetYDataPnt(uint setNr){ return dataSetMap[setNr].yDataPnt(); };
    QVector<double> dataSetXData(uint setNr){ return dataSetMap[setNr].xData(); };
    QVector<double> dataSetYData(uint setNr){ return dataSetMap[setNr].yData(); };
    double dataSetXMinVal(uint setNr){ return dataSetMap[setNr].xMinVal(); };
    double dataSetXMaxVal(uint setNr){ return dataSetMap[setNr].xMaxVal(); };
    double dataSetYMinVal(uint setNr){ return dataSetMap[setNr].yMinVal(); };
    double dataSetYMaxVal(uint setNr){ return dataSetMap[setNr].yMaxVal(); };
    int dataSetDataType(uint setNr){ return dataSetMap[setNr].dataType(); };
    int dataSetPopulationNr(uint setNr){ return dataSetMap[setNr].populationNr(); };
    QwtPlot::Axis dataSetXAxis(uint setNr){ return dataSetMap[setNr].xAxis(); };
    QwtPlot::Axis dataSetYAxis(uint setNr){ return dataSetMap[setNr].yAxis(); };
    QString dataSetLegendLabel(uint setNr){ return dataSetMap[setNr].legendLabel(); };
    QwtSymbol *dataSetCurveSymbol(uint setNr){ return dataSetMap[setNr].curveSymbol(); };
    QPen dataSetLinePen(uint setNr){ return dataSetMap[setNr].linePen(); };
    QwtPlotCurve::CurveStyle dataSetCurveStyle(uint setNr){ return dataSetMap[setNr].curveStyle(); };
    int dataSetHistNumBars(uint setNr){ return dataSetMap[setNr].histNumBars(); };
    EBTPlotCurve *dataSetCurveID(uint setNr){ return dataSetMap[setNr].curveID(); };
    int dataSetOldSize(uint setNr){ return dataSetMap[setNr].oldSize(); };
    int dataSetSize(uint setNr){ return dataSetMap[setNr].size(); };
    bool dataSetIsNew(uint setNr){ return dataSetMap[setNr].isNew(); };
    void setXPos(int value){ xpos = value; };
    void setYPos(int value){ ypos = value; };
    void setXSize(int value){ xsize = value; };
    void setYSize(int value){ ysize = value; };
    void setPlotTitle(QString value){ plottitle = value; };
    void setPlotTitleFont(QFont value){ plottitlefont = value; };
    void setShowGrid(bool value){ showgrid = value; };
    void setShowFineGrid(bool value){ showfinegrid = value; };
    void setShowLegend(bool value){ showlegend = value; };
    void setLegendFramed(bool value) { legendframed = value; };
    void setLegendFrameWidth(int value) { legendframewidth = value; };
    void setLegendPosition(int value) { legendposition = value; };
    void setLegendXPosition(double value) { legendxposition = value; };
    void setLegendYPosition(double value) { legendyposition = value; };
    void setShowTimestamp(bool value){ showtimestamp = value; };
    void setShowDescription(bool value){ showdescription = value; };
    void setDefaultDescription(bool value){ defaultdescription = value; };
    void setPlotDescription(QString value){ plotdescription = value; };
    void setLegendFont(QFont value){ legendfont = value; };
    void setAllSetsSameAs(int value){ allsetssameas = value; };

    void setAxisLogarithmic(QwtPlot::Axis a, bool value){ axisMap[a].logarithmic = value; };
    void setAxisTitle(QwtPlot::Axis a, QString value){ axisMap[a].title = value; };
    void setAxisTitleFont(QwtPlot::Axis a, QFont value){ axisMap[a].titleFont = value; };
    void setAxisAutoscale(QwtPlot::Axis a, bool value){ axisMap[a].autoscale = value; };
    void setAxisMinVal(QwtPlot::Axis a, double value){ axisMap[a].minVal = value; };
    void setAxisMaxVal(QwtPlot::Axis a, double value){ axisMap[a].maxVal = value; };

    void setDataSetXCol(uint setNr, uint value){ dataSetMap[setNr].setXCol(value); };
    void setDataSetYCol(uint setNr, uint value){ dataSetMap[setNr].setYCol(value); };
    void setDataSetDataType(uint setNr, int value){ dataSetMap[setNr].setDataType(value); };
    void setDataSetPopulationNr(uint setNr, int value){ dataSetMap[setNr].setPopulationNr(value); };
    void setDataSetXAxis(uint setNr, QwtPlot::Axis value){ dataSetMap[setNr].setXAxis(value); };
    void setDataSetYAxis(uint setNr, QwtPlot::Axis value){ dataSetMap[setNr].setYAxis(value); };
    void setDataSetLegendLabel(uint setNr, QString value){ dataSetMap[setNr].setLegendLabel(value); };
    void setDataSetCurveSymbol(uint setNr, QwtSymbol *value){ dataSetMap[setNr].setCurveSymbol(value); };
    void setDataSetLinePen(uint setNr, QPen value){ dataSetMap[setNr].setLinePen(value); };
    void setDataSetCurveStyle(uint setNr, QwtPlotCurve::CurveStyle value){ dataSetMap[setNr].setCurveStyle(value); };
    void setDataSetHistNumBars(uint setNr, int value){ dataSetMap[setNr].setHistNumBars(value); };
    void setDataSetCurveID(uint setNr, EBTPlotCurve * value){ dataSetMap[setNr].setCurveID(value); };
    void setDataSetOldSize(uint setNr, int value){ dataSetMap[setNr].setOldSize(value); };
    void setDataSetIsNew(uint setNr, bool value){ dataSetMap[setNr].setIsNew(value); };
    void setDataSetXData(uint setNr, const EBTData &value){ dataSetMap[setNr].setXData(value); };
    void setDataSetYData(uint setNr, const EBTData &value){ dataSetMap[setNr].setYData(value); };
    void dataSetXAppend(uint setNr, double value){ dataSetMap[setNr].xAppend(value); };
    void dataSetYAppend(uint setNr, double value){ dataSetMap[setNr].yAppend(value); };
};

#endif	// EBTGRSETTINGS_H
