#include "ebtgrfile.h"
#include "ebtgraphsettings.h"

EBTGrFile::EBTGrFile()
{
    setDescriptionText("Graph layout");
    isOutputGraph = TRUE; 			// has to be initialized
}

EBTGrFile::~EBTGrFile()
{
}

void EBTGrFile::load()
{
    settings.clearDataSetMap();
    EBTXMLFile::load();
}

void EBTGrFile::readElement(const QDomElement &element)
{
  if (element.tagName() == "general")
    {
      settings.setXPos( readIntAttribute(element, "xpos"));
      settings.setXSize( readIntAttribute(element, "xsize" /*, &attributeFound*/));
      settings.setYPos( readIntAttribute(element, "ypos"));
      settings.setYSize( readIntAttribute(element, "ysize"));
      settings.setPlotTitle( readStringAttribute(element, "title")); 
      settings.setPlotTitleFont( readFontAttribute(element, "titlefont"));
      settings.setShowGrid( readBoolAttribute(element, "showgrid"));
      settings.setShowFineGrid( readBoolAttribute(element, "showfinegrid"));
      settings.setAllSetsSameAs( readIntAttribute(element, "allsetssameas"));
    }
  else if (element.tagName() == "labeling")
    readLabelsNode(element);
  else if (element.tagName() == "axis")
    readAxisNode(element);
  else if (element.tagName() == "dataset")
    readDataSetNode(element);
    
}


QDomDocument EBTGrFile::createDomDoc(const QString &docName)
{
    QDomDocument domDoc(docName);
    QDomElement root = domDoc.createElement("ebtgr");
    domDoc.appendChild(root);
    
    QDomElement generalNode = domDoc.createElement("general");
    
    //    addAttribute(domDoc, generalNode, "outputgraph", settings.outGraph);
    addAttribute(domDoc, generalNode, "xpos", settings.xPos());
    addAttribute(domDoc, generalNode, "xsize", settings.xSize());
    addAttribute(domDoc, generalNode, "ypos", settings.yPos());
    addAttribute(domDoc, generalNode, "ysize", settings.ySize());
    addAttribute(domDoc, generalNode, "title", settings.plotTitle());
    addAttribute(domDoc, generalNode, "titlefont", settings.plotTitleFont());
    addAttribute(domDoc, generalNode, "showgrid", settings.showGrid());
    addAttribute(domDoc, generalNode, "showfinegrid", settings.showFineGrid());
    addAttribute(domDoc, generalNode, "allsetssameas", settings.allSetsSameAs());
    
    root.appendChild(generalNode);
    root.appendChild( createLabelsNode(domDoc) );

    for(QwtPlot::Axis a = QwtPlot::yLeft; a < QwtPlot::axisCnt; a = QwtPlot::Axis(a + 1))
	root.appendChild( createAxisNode(domDoc, a) );
    
    for(uint dataSetNr = 0; dataSetNr < settings.dataSetMapSize(); dataSetNr++)
	root.appendChild( createDataSetNode(domDoc, dataSetNr) );
    
    return domDoc;
}


QDomElement EBTGrFile::createLabelsNode(QDomDocument &doc)
{
  QDomElement legendNode = doc.createElement("labeling");

  addAttribute(doc, legendNode, "showlegend", settings.showLegend());
  addAttribute(doc, legendNode, "framelegend", settings.legendFramed());
  addAttribute(doc, legendNode, "framewidth", settings.legendFrameWidth());
  addAttribute(doc, legendNode, "legendposition", (int)settings.legendPosition());
  addAttribute(doc, legendNode, "legendxposition", settings.legendXPosition());
  addAttribute(doc, legendNode, "legendyposition", settings.legendYPosition());
  addAttribute(doc, legendNode, "showtimestamp", settings.showTimestamp());
  addAttribute(doc, legendNode, "showdescription", settings.showDescription());
  addAttribute(doc, legendNode, "defaultdescription", settings.defaultDescription());
  addAttribute(doc, legendNode, "description", settings.plotDescription());
  addAttribute(doc, legendNode, "legendfont", settings.legendFont());

  return legendNode;
}

QDomElement EBTGrFile::createAxisNode(QDomDocument &doc, const QwtPlot::Axis &a)
{
    QDomElement axisNode = doc.createElement("axis");
    
    addAttribute(doc, axisNode, "number", (int)a);
    addAttribute(doc, axisNode, "autoscale", settings.axisAutoscale(a));
    addAttribute(doc, axisNode, "minval", settings.axisMinVal(a));
    addAttribute(doc, axisNode, "maxval", settings.axisMaxVal(a));
    addAttribute(doc, axisNode, "logarithmic", settings.axisLogarithmic(a));
    addAttribute(doc, axisNode, "title", settings.axisTitle(a));
    addAttribute(doc, axisNode, "titlefont", settings.axisTitleFont(a));
    
    return axisNode;
}

QDomElement EBTGrFile::createDataSetNode(QDomDocument &doc, const uint &dataSetNr)
{
    QDomElement setNode = doc.createElement("dataset");
    
    addAttribute(doc, setNode, "xcol", (int)settings.dataSetXCol(dataSetNr));
    addAttribute(doc, setNode, "ycol", (int)settings.dataSetYCol(dataSetNr));
    addAttribute(doc, setNode, "datatype", settings.dataSetDataType(dataSetNr));
    addAttribute(doc, setNode, "population", settings.dataSetPopulationNr(dataSetNr));
    addAttribute(doc, setNode, "xaxis", settings.dataSetXAxis(dataSetNr));
    addAttribute(doc, setNode, "yaxis", settings.dataSetYAxis(dataSetNr));
    addAttribute(doc, setNode, "legendlabel", settings.dataSetLegendLabel(dataSetNr));
    addAttribute(doc, setNode, "symbol", settings.dataSetCurveSymbol(dataSetNr));
    addAttribute(doc, setNode, "line", settings.dataSetLinePen(dataSetNr));
    addAttribute(doc, setNode, "curvestyle", settings.dataSetCurveStyle(dataSetNr));
    addAttribute(doc, setNode, "histnumbars", settings.dataSetHistNumBars(dataSetNr));
 
    return setNode;
}


void EBTGrFile::readLabelsNode(const QDomElement &el)
{
  settings.setShowLegend( readBoolAttribute(el, "showlegend"));
  settings.setLegendFramed( readBoolAttribute(el, "framelegend"));
  settings.setLegendFrameWidth( readIntAttribute(el, "framewidth"));
  settings.setLegendPosition( readIntAttribute(el, "legendposition"));
  settings.setLegendXPosition( readDoubleAttribute(el, "legendxposition"));
  settings.setLegendYPosition( readDoubleAttribute(el, "legendyposition"));
  if (settings.legendPosition() != (QwtPlot::TopLegend+1))
    if ((settings.legendPosition() != QwtPlot::BottomLegend) && (settings.legendPosition() != QwtPlot::RightLegend) &&
	(settings.legendPosition() != QwtPlot::TopLegend))
      settings.setLegendPosition( QwtPlot::BottomLegend );
  settings.setShowTimestamp( readBoolAttribute(el, "showtimestamp"));
  settings.setShowDescription( readBoolAttribute(el, "showdescription"));
  settings.setDefaultDescription( readBoolAttribute(el, "defaultdescription"));
  settings.setPlotDescription( readStringAttribute(el, "description"));
  settings.setLegendFont( readFontAttribute(el, "legendfont"));
}

void EBTGrFile::readAxisNode(const QDomElement &el)
{
    QwtPlot::Axis a = (QwtPlot::Axis) readIntAttribute(el, "number");
    settings.setAxisLogarithmic( a, readBoolAttribute(el, "logarithmic"));
    settings.setAxisTitle( a, readStringAttribute(el, "title"));
    settings.setAxisTitleFont( a, readFontAttribute(el, "titlefont"));
    settings.setAxisAutoscale( a, readBoolAttribute(el, "autoscale"));
    settings.setAxisMinVal( a, readDoubleAttribute(el, "minval"));
    settings.setAxisMaxVal( a, readDoubleAttribute(el, "maxval"));
}

void EBTGrFile::readDataSetNode(const QDomElement &el)
{
    uint ii = settings.addDataSet();
    settings.setDataSetXCol(ii, (uint)readIntAttribute(el, "xcol"));
    settings.setDataSetYCol(ii, (uint)readIntAttribute(el, "ycol"));
    int datatype = readIntAttribute(el, "datatype", &attributeFound);
    if(datatype == COMPUTEDVALUES || datatype == FIXEDVALUES || datatype == OUTPUTVALUES)
	settings.setDataSetDataType(ii, datatype);
    else
	settings.setDataSetDataType(ii, OUTPUTVALUES);
    settings.setDataSetPopulationNr(ii, readIntAttribute(el, "population"));
    settings.setDataSetXAxis(ii, (QwtPlot::Axis)readIntAttribute(el, "xaxis"));
    settings.setDataSetYAxis(ii, (QwtPlot::Axis)readIntAttribute(el, "yaxis"));
    settings.setDataSetLegendLabel(ii, readStringAttribute(el, "legendlabel"));
    QwtSymbol *symbolRead = readSymbolAttribute(el, "symbol");
    settings.setDataSetCurveSymbol(ii, symbolRead);
    delete symbolRead;
    settings.setDataSetLinePen(ii, readPenAttribute(el, "line"));
    settings.setDataSetCurveStyle(ii, (QwtPlotCurve::CurveStyle)readIntAttribute(el, "curvestyle", &attributeFound));
    if(!attributeFound) 
	settings.setDataSetCurveStyle(ii, QwtPlotCurve::Lines);
    settings.setDataSetHistNumBars(ii, readIntAttribute(el, "histnumbars"));
    if(!attributeFound) 
	settings.setDataSetHistNumBars(ii, 10);
}

EBTOGrFile::EBTOGrFile()
{
    setFileExtension("ebtogr");
    setTemplateFile("templates/tmpl.ebtogr");
    setDescriptionText("output graph");
    setSaveText(descriptionText() + " (*." + fileExtension() + ")") ;
    setOpenText(saveText());
    setGraphType(TRUE);
}


EBTOGrFile::~EBTOGrFile()
{
}


EBTSGrFile::EBTSGrFile()
{
    setFileExtension("ebtsgr");
    setTemplateFile("templates/tmpl.ebtsgr");
    setDescriptionText("state graph");
    setSaveText(descriptionText() + " (*." + fileExtension() + ")") ;
    setOpenText(saveText());
    setGraphType(FALSE);
}

EBTSGrFile::~EBTSGrFile()
{
}
