#ifndef EBTMODEL_H
#define EBTMODEL_H

#include <string>
#include <qlibrary.h>
#include "ebttextfile.h"

class EBTModel : public EBTFile
{
private:
    QLibrary *libHandle;
    void messageLoadError(QString handle);
    typedef int (*functionType1)(int, char **); 
    typedef int (*functionType2)(void);
    typedef int (*functionType3)(int);
    typedef char* (*functionType4)(void);
    typedef double* (*functionType5)(void);
    typedef double** (*functionType6)(void);
    typedef int* (*functionType7)(void);
    typedef char* (*functionType8)(int);
    
    functionType1 startup;
    functionType2 cycleinit;
    functionType2 cyclestep;
    functionType3 shutdown;
    functionType3 handlesignal;
    
    functionType2 outputvarnum;
    functionType2 environdim;
    functionType2 populationnum;
    functionType2 istatedim;
    functionType2 iconstdim;
    functionType2 parameternum;
    functionType4 errormsg;
    
    functionType5 output;
    functionType5 env;	
    functionType7 cohortnum;
    functionType6 pop;
    functionType6 popidcard;
    functionType8 statelabel;
    
public:
    EBTModel();
    virtual ~EBTModel();
    
    bool open();
    bool save();
    void setFileName(const QString& str);
    void libUnload();
    bool initHandles();
    bool libLoaded(){return libHandle != NULL;};
    int startUp(int numArgs, char **theArgs);
    int cycleInit();
    int cycleStep();
    int shutDown(int flag);
    int handleSignal(int signl);	////////////////////////////////////////////////
    uint outputVarNum(){return (*outputvarnum)();};
    uint environDim(){return (*environdim)();};
    uint populationNum(){return (*populationnum)();};
    uint iStateDim(){return (*istatedim)();};
    uint iConstDim(){return (*iconstdim)();};
    uint parameterNum(){return (*parameternum)();};
    double* outputData(){return (output)();};
    double* environment(){return (env)();};
    int* cohortNum(){return (cohortnum)();};
    double** population(){return (pop)();};
    double** popIDcard(){return (popidcard)();};
    QString errorMsg(){return QString::fromLatin1((*errormsg)(), -1);};
    QString stateLabel(int popnr);
    QString soName();
    QString exportState();
    
    EBTTextFile hFile;
    EBTTextFile cFile;
};

#endif // EBTMODEL_H
