#include "ebtprfile.h"
#include "qmessagebox.h"

EBTPrFile::EBTPrFile()
{
    setTemplateFile("templates/tmpl.ebtpr");
    setFileExtension("ebtpr");
    setDescriptionText("EBT Project");
    setSaveText(descriptionText() + " (*." + fileExtension() + ")") ;
    setOpenText(saveText());
}

EBTPrFile::~EBTPrFile()
{
}

void EBTPrFile::load()
{
    project.clearMap();
//    project.graphMap.clear();
    EBTXMLFile::load();
}

void EBTPrFile::readElement(const QDomElement &element)
{
    if (element.tagName() == "general")
    {
	project.setName(readStringAttribute(element, "name"));
	project.setDescription(readStringAttribute(element, "description"));
	project.setSaveTime(readStringAttribute(element, "time"));
	project.setArguments(readStringAttribute(element, "arguments"));
    }
    else if (element.tagName() == "fileref")
	readFileNode(element);
//    else if (element.tagName() == "graphref")
//	readGraphNode(element);    
}

QDomDocument EBTPrFile::createDomDoc(const QString &docName)
{
    QDomDocument domDoc(docName);
    QDomElement root = domDoc.createElement("ebtproj");
    domDoc.appendChild(root);
    
    QDomElement generalNode = domDoc.createElement("general");
    
    addAttribute(domDoc, generalNode, "name", project.projectName());
    addAttribute(domDoc, generalNode, "description", project.projectDescription());
    addAttribute(domDoc, generalNode, "time", project.saveTime());
    addAttribute(domDoc, generalNode, "arguments", project.runtimeArguments());
    
    root.appendChild(generalNode);
    
    for(uint fileRefNr = 0; fileRefNr < project.fileMapSize(); fileRefNr++)
	root.appendChild(createFileNode(domDoc, fileRefNr));

    return domDoc;
}

QDomElement EBTPrFile::createFileNode(QDomDocument &doc, const uint fileRefNr)
{
    QDomElement fileNode = doc.createElement("fileref");
    
    addAttribute(doc, fileNode, "type", (int)project.fileRefType(fileRefNr));
//    QString fName = QFileInfo(fr.name).dirPath(TRUE);
    QString fName = project.fileRefName(fileRefNr);
    if(fName.startsWith(EBTPrFile::dirPath()))
    {
	fName = fName.right(fName.length() - EBTPrFile::dirPath().length() - 1);
	addAttribute(doc, fileNode, "name", fName);
    }
    else
	addAttribute(doc, fileNode, "name", fName);
    
//    addAttribute(doc, fileNode, "shown", fr.shown);
    if ((project.fileRefType(fileRefNr) == EBTOGR) ||
	(project.fileRefType(fileRefNr) == EBTSGR))
      {
	addAttribute(doc, fileNode, "xpos", project.fileRefXPos(fileRefNr));
	addAttribute(doc, fileNode, "ypos", project.fileRefYPos(fileRefNr));
	addAttribute(doc, fileNode, "width", project.fileRefWidth(fileRefNr));
	addAttribute(doc, fileNode, "height", project.fileRefHeight(fileRefNr));
      }
//    addAttribute(doc, fileNode, "xsize", fr.xSize);
//    addAttribute(doc, fileNode, "ysize", fr.ySize);
 
    return fileNode;
}


void EBTPrFile::readFileNode(const QDomElement &el)
{
    uint ii = project.addFileRef();
    project.setFileRefType(ii, (fileType)readIntAttribute(el, "type"));
    
    QString fName = readStringAttribute(el, "name");
    if(QFileInfo(fName).isRelative())
	fName = EBTPrFile::dirPath() + "/" + fName;
    project.setFileRefName(ii, fName);
//    project.fileMap[ii].shown = readBoolAttribute(el, "shown");
    if ((project.fileRefType(ii) == EBTOGR) ||
	(project.fileRefType(ii) == EBTSGR))
      {
	project.setFileRefXPos(ii, readIntAttribute(el, "xpos"));
	project.setFileRefYPos(ii, readIntAttribute(el, "ypos"));
	project.setFileRefWidth(ii, readIntAttribute(el, "width"));
	project.setFileRefHeight(ii, readIntAttribute(el, "height"));
      }
//    project.fileMap[ii].xSize = readIntAttribute(el, "xsize");
//    project.fileMap[ii].ySize = readIntAttribute(el, "ysize");
}
