#include <QFileDialog>
#include <qmessagebox.h>
#include "ebtrun.h"

EBTRun::EBTRun()
{
    setDescriptionText("Run");
    setOpenText("Run (*.cvf)") ;
    
    cvfFile.setTemplateFile("templates/tmpl.cvf");
    cvfFile.setFileExtension("cvf");
    cvfFile.setDescriptionText("Control variable file");
    cvfFile.setSaveText(cvfFile.descriptionText() + " (*." + cvfFile.fileExtension() + ")") ;
    cvfFile.setOpenText(cvfFile.saveText());
    
    isfFile.setTemplateFile("templates/tmpl.isf");
    isfFile.setFileExtension("isf");
    isfFile.setDescriptionText("Initial state file");
    isfFile.setSaveText(isfFile.descriptionText() + " (*." + isfFile.fileExtension() + ")") ;
    isfFile.setOpenText(isfFile.saveText());
}    

EBTRun::~EBTRun()
{
}

bool EBTRun::open()
{
    QString fName = QFileDialog::getOpenFileName(NULL, "EBTtool -- file open", dirPath(), openText());
    
    if (!fName.isEmpty())
    {
	QFileInfo fi(fName);
	QString basename = fi.absolutePath() + "/" + fi.completeBaseName();
	
	//waarschuwing als  .isf ontbreekt
	QString isfFilename = basename + "." + isfFile.fileExtension();
	if(!QFile::exists(isfFilename))
	{
	    QMessageBox::warning(0, EBTCAPTION + "File missing",
				 QString("The file %1 does not exist.\nExpecting initialization in UserInit()!")
				 .arg(isfFilename));
	//    return FALSE;
	}
	/////////////////////
	setFileName(basename);
	return TRUE;
    }
    else
	return FALSE;
    
}

bool EBTRun::save()
{
    if(!isfFile.save())
	return FALSE;
    if(!cvfFile.save())
	return FALSE;
    return TRUE;
}


void EBTRun::setFileName(const QString& str)
{
    EBTFile::setFileName(str);  

    isfFile.setFileName(str + "." + isfFile.fileExtension());

    QString isfFilename = str + "." + isfFile.fileExtension();

    if(str.isEmpty() || str.isNull() || !QFile::exists(isfFilename))
      isfFile.setFileName("");

    cvfFile.setFileName(str + "." + cvfFile.fileExtension());
}
