#include <float.h>
#include <math.h>
#include "ebtdefines.h"
#include "integrator/ebttooldefs.h"
#include "ebttool.h"

#include <qvariant.h>
#include <qimage.h>
#include <qpixmap.h>

// #include "qstring.h"
#include "qstatusbar.h"
// #include "qmessagebox.h"
#include "qapplication.h"
#include "qfileinfo.h"
#include "qinputdialog.h"
#include "qdir.h"
#include "compileform.h"
#include "projectform.h"
#include "qdatetime.h"
#include <QSettings>
#include "QFileDialog"
#include "QMessageBox"

#include <QMouseEvent>
#include <QCloseEvent>
#include <QEvent>
#include <QList>
#include <QString>
#include <QtGlobal>
#include <QProcessEnvironment>

#ifdef DEBUG
#include <QtDebug>
#endif

#include "ebtfiledefs.h"

#ifndef _MSC_VER
#include <iostream>
using namespace std;
#else
#include "string.h"
#endif

#define STRCPY strcpy

// Signal handling; tzt afhankelijk van HAS_SIGNALS:
#include <signal.h>

#if   (defined(Q_OS_LINUX) || defined(Q_OS_MACOS) || (defined(Q_OS_WIN) && defined(__MINGW32__)))

// FPU error control
#include <fenv.h>
#define testFPU()		fetestexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW)
#define clearFPU()		feclearexcept(FE_ALL_EXCEPT)
#define testExcepts(x)		(x & (FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW))

#elif defined(Q_OS_WIN)

// FPU error control
#include <float.h>
#define testFPU()		_status87( /*EM_INVALID | EM_ZERODIVIDE | EM_OVERFLOW*/ )
#define clearFPU()		_clear87()
#define testExcepts(x)		(x & (EM_ZERODIVIDE | EM_INVALID | EM_OVERFLOW))

#endif

#if defined(Q_OS_MACOS)

#include <CoreFoundation/CFBundle.h>
#include <CoreFoundation/CFString.h>

clock_t		lastUpdate, currentTime;

#endif


/*
 *  Constructs a EBTtool as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 */

EBTtool::EBTtool(QWidget* parent, Qt::WindowFlags fl)
    : QMainWindow(parent, fl)
{
    setupUi(this);
    stateBrowseComboBox->setMinimumContentsLength(1);
    stateBrowseComboBox->clear();
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "10", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "9", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "8", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "7", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "6", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "5", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "4", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "3", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "2", 0));
    stateBrowseComboBox->addItem(QApplication::translate("EBTtool", "1", 0));
    stateBrowseComboBox->setProperty("toolTip", QVariant(QApplication::translate("EBTtool", "Browse speed", 0)));
    stateBrowseComboBox->setCurrentIndex(5);

    QSizePolicy sizePolicy1(static_cast<QSizePolicy::Policy>(1), static_cast<QSizePolicy::Policy>(0));
    sizePolicy1.setHorizontalStretch(0);
    sizePolicy1.setVerticalStretch(0);
    sizePolicy1.setHeightForWidth(stateBrowseScrollBar->sizePolicy().hasHeightForWidth());
    stateBrowseScrollBar->setSizePolicy(sizePolicy1);
    stateBrowseScrollBar->setMinimumSize(QSize(350, 0));
    stateBrowseScrollBar->setMouseTracking(false);
    stateBrowseScrollBar->setTracking(true);
#if defined (Q_OS_MACOS)
    setUnifiedTitleAndToolBarOnMac (TRUE);
#endif
    errorMessage = NULL;

    hideCSBControls(TRUE);

    (void)statusBar();
    init();
    activateWindow();
}


/*
 *  Destroys the object and frees any allocated resources
 */

EBTtool::~EBTtool()
{
  destroy();
  // no need to delete child widgets, Qt does it all for us
}


/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */

void EBTtool::languageChange()
{
    retranslateUi(this);
}


void EBTtool::init()
{
    loadSettings();
    initFiles();
    initVars();

    checkEBTDir();

    updateView();
    updateMenu(FALSE);
}

void EBTtool::hideEvent ( QHideEvent * e )
{
  QWidget::hideEvent ( e );

  for (int gg = 0; gg < sgList->size(); ++gg) sgList->at(gg)->hide();

  for (int gg = 0; gg < ogList->size(); ++gg) ogList->at(gg)->hide();

  for (int gg = 0; gg < teList->size(); ++gg) teList->at(gg)->hide();
}

void EBTtool::showEvent ( QShowEvent * e )
{
#ifdef DEBUG
  qDebug("EBTtool::showEvent()");
#endif
    QWidget::showEvent ( e );
    for (int gg = 0; gg < sgList->size(); ++gg) if (!sgList->at(gg)->getHideFlag()) sgList->at(gg)->show();
    for (int gg = 0; gg < ogList->size(); ++gg) if (!ogList->at(gg)->getHideFlag()) ogList->at(gg)->show();
    for (int gg = 0; gg < teList->size(); ++gg) teList->at(gg)->show();
    updateGraphMenu();
}

void EBTtool::hideCSBControls(bool allHidden)
{
  stateBack->setVisible(!allHidden);
  statePause->setVisible(!allHidden);
  stateForward->setVisible(!allHidden);
  stateBrowseComboBox->setVisible(!allHidden);
  stateBrowseScrollBar->setVisible(!allHidden);
}

void EBTtool::disableCSBControls(bool allDisabled)
{
  stateBack->setDisabled(allDisabled);
  statePause->setDisabled(allDisabled);
  stateForward->setDisabled(allDisabled);
  stateBrowseComboBox->setDisabled(allDisabled);
  stateBrowseScrollBar->setDisabled(allDisabled);
}

void EBTtool::fileNew(EBTFile& ef)
{
    ef.createNew();
    updateView();
    updateMenu(FALSE);
}


void EBTtool::fileOpen(EBTFile& ef)
{
    statusBar()->showMessage("Opening file...");
    if(!ef.open())
  statusBar()->showMessage("File open abandoned", 2000);
    else
  statusBar()->showMessage("Ready");
    updateMenu(FALSE);
}


void EBTtool::fileSaveAs(EBTFile& ef)
{
    statusBar()->showMessage("Saving file...");
    if(!ef.saveAs())
  statusBar()->showMessage("Saving abandoned", 2000);
    else
  statusBar()->showMessage("Ready");
}


QString EBTtool::fileCopyAs(EBTFile& ef, const QString & nname)
{
  QString nn;

  statusBar()->showMessage("Copying file...");
  nn = ef.copyAs(nname);

  if(nn.isNull() || nn.isEmpty())
    statusBar()->showMessage("Copying abandoned", 2000);
  else
    statusBar()->showMessage("Ready");
  return nn;
}


void EBTtool::fileExit()
{
    saveSettings();
    if(okToClear())
  qApp->exit(0);
}


void EBTtool::helpIndex()
{

}


void EBTtool::helpContents()
{

}


void EBTtool::helpAbout()
{
    QMessageBox::information(this, EBTCAPTION + "About",
           QString("EBTtool version %1").arg(EBTVERSION) + QString("\nAuthors: Andre M. de Roos, Arjan van de Merwe"));
}

void EBTtool::helpAboutQt()
{
    QMessageBox::aboutQt(this, EBTCAPTION + "About Qt");
}

void EBTtool::fileOutputValuesOpen()
{
    QString oldFileName = outFile.absFilePath();
    fileOpen(outFile);
    if (outFile.fileName().isEmpty())
  return;

    if ((outputData.count() > 0) && (outFile.absFilePath() != oldFileName))
      {
  QMessageBox msgBox;

  msgBox.setWindowTitle(EBTCAPTION + "Append or replace data?    ");
  msgBox.setIcon(QMessageBox::Question);
  msgBox.setText(QString("Append new data to current,\noutput data or replace them?"));
  QPushButton *appendButton  = msgBox.addButton(tr("Append"), QMessageBox::ActionRole);
  QPushButton *replaceButton = msgBox.addButton(tr("Replace"), QMessageBox::ActionRole);
  msgBox.addButton(QMessageBox::Abort);

  msgBox.exec();

  if (msgBox.clickedButton() == appendButton)
    loadOutputValues(APPENDDATA);
  else if (msgBox.clickedButton() == replaceButton)
    loadOutputValues(REPLACEDATA);
  else
    outFile.setFileName(oldFileName);
      }
    else loadOutputValues(REPLACEDATA);

    getOutFileSize();
    updateView();
}


void EBTtool::fileComputedValuesOpen()
{
    QString oldFileName = compFile.absFilePath();
    fileOpen(compFile);
    if(compFile.fileName().isEmpty())
  return;

    if ((computedData.count() > 0) && (compFile.absFilePath() != oldFileName))
      {
  QMessageBox msgBox;

  msgBox.setWindowTitle(EBTCAPTION + "Append or replace data?    ");
  msgBox.setIcon(QMessageBox::Question);
  msgBox.setText(QString("Append new data to current,\ncomputed data or replace them?"));
  QPushButton *appendButton  = msgBox.addButton(tr("Append"), QMessageBox::ActionRole);
  QPushButton *replaceButton = msgBox.addButton(tr("Replace"), QMessageBox::ActionRole);
  msgBox.addButton(QMessageBox::Abort);

  msgBox.exec();

  if (msgBox.clickedButton() == appendButton)
    loadComputedValues(APPENDDATA);
  else if (msgBox.clickedButton() == replaceButton)
    loadComputedValues(REPLACEDATA);
  else
    compFile.setFileName(oldFileName);
      }
    else loadComputedValues(REPLACEDATA);

    updateView();
}


void EBTtool::fileFixedValuesOpen()
{
    QString oldFileName = fixFile.absFilePath();
    fileOpen(fixFile);
    if(fixFile.fileName().isEmpty())
  return;
    if ((fixedData.count() > 0) && (fixFile.absFilePath() != oldFileName))
      {
  QMessageBox msgBox;

  msgBox.setWindowTitle(EBTCAPTION + "Append or replace data?    ");
  msgBox.setIcon(QMessageBox::Question);
  msgBox.setText(QString("Append new data to current,\nfixed data or replace them?"));
  QPushButton *appendButton  = msgBox.addButton(tr("Append"), QMessageBox::ActionRole);
  QPushButton *replaceButton = msgBox.addButton(tr("Replace"), QMessageBox::ActionRole);
  msgBox.addButton(QMessageBox::Abort);

  msgBox.exec();

  if (msgBox.clickedButton() == appendButton)
    loadFixedValues(APPENDDATA);
  else if (msgBox.clickedButton() == replaceButton)
    loadFixedValues(REPLACEDATA);
  else
    fixFile.setFileName(oldFileName);
      }
    else loadFixedValues(REPLACEDATA);

    updateView();
}


void EBTtool::fileRereadValues()
{
    if(!outFile.fileName().isEmpty())
  loadOutputValues(REPLACEDATA);
    if(!fixFile.fileName().isEmpty())
  loadFixedValues(REPLACEDATA);
    if(!compFile.fileName().isEmpty())
  loadComputedValues(REPLACEDATA);
}


void EBTtool::graphNewGraphSlot(bool outputGraph)
{
    if(outputGraph)
  graphOutputGraphNew();
    else
  graphStateGraphNew();
#ifdef DEBUG
    qDebug("graphNewGraphSlot() called");
#endif
}


void EBTtool::graphOutputGraphOpen()
{
  EBTOGrFile *ebtogrFile = new EBTOGrFile;

  if(!outputGraphDirPath.isEmpty())
    ebtogrFile->setDirPath(outputGraphDirPath);
  else if(!stateGraphDirPath.isEmpty())
    ebtogrFile->setDirPath(stateGraphDirPath);
  fileOpen(*ebtogrFile);
  if(ebtogrFile->fileName().isEmpty())
    {
      delete ebtogrFile;
      return;
    }

  outputGraphDirPath = ebtogrFile->dirPath();
  graphOutputGraph(*ebtogrFile);
}


void EBTtool::graphOutputGraphNew()
{
  EBTOGrFile *ebtogrFile = new EBTOGrFile;
  graphOutputGraph(*ebtogrFile);
}

void EBTtool::graphOutputGraph(EBTOGrFile & graphFile)
{
  EBTGraphForm *ograph = new EBTGraphForm(this);
  ogList->append(ograph);
  ograph->show();
  ograph->setFile(graphFile);

  updateGraphMenu();
  statusBar()->showMessage("Ready");
}

void EBTtool::graphOutputGraph(EBTOGrFile & graphFile, int xp, int yp, int wd, int hg)
{
  EBTGraphForm *ograph = new EBTGraphForm(this);
  ogList->append(ograph);
  ograph->setFile(graphFile);
  ograph->show();
  if ((wd > 0) && (hg > 0)) ograph->resize(wd, hg);
  if ((xp > 0) && (yp > 0)) ograph->move(xp, yp);

  updateGraphMenu();
  statusBar()->showMessage("Ready");
}

void EBTtool::graphStateGraphOpen()
{
  EBTSGrFile *ebtsgrFile = new EBTSGrFile;

  if(!stateGraphDirPath.isEmpty())
    ebtsgrFile->setDirPath(stateGraphDirPath);
  else if(!outputGraphDirPath.isEmpty())
    ebtsgrFile->setDirPath(outputGraphDirPath);
  fileOpen(*ebtsgrFile);
  if(ebtsgrFile->fileName().isEmpty())
    {
      delete ebtsgrFile;
      return;
    }
  stateGraphDirPath = ebtsgrFile->dirPath();
  graphStateGraph(*ebtsgrFile);
}


void EBTtool::graphStateGraphNew()
{
  EBTSGrFile *ebtsgrFile = new EBTSGrFile;
  graphStateGraph(*ebtsgrFile);
}


void EBTtool::graphStateGraph(EBTSGrFile & graphFile)
{
  EBTGraphForm *sgraph = new EBTGraphForm(this, Qt::Widget);

  sgList->append(sgraph);
  sgraph->show();
  sgraph->setFile(graphFile);

  updateGraphMenu();
  statusBar()->showMessage("Ready");
}

void EBTtool::graphStateGraph(EBTSGrFile & graphFile, int xp, int yp, int wd, int hg)
{
  EBTGraphForm *sgraph = new EBTGraphForm(this);
  sgList->append(sgraph);
  sgraph->setFile(graphFile);
  sgraph->show();
  if ((wd > 0) && (hg > 0)) sgraph->resize(wd, hg);
  if ((xp > 0) && (yp > 0)) sgraph->move(xp, yp);

  updateGraphMenu();
  statusBar()->showMessage("Ready");
}


void EBTtool::modelCompileSlot()
{
    modelCompile(TRUE);
}


void EBTtool::modelDefIDEdit()
{
    defidcard.resetFileName();
    fileEdit(defidcard.cFile, FALSE);
}


void EBTtool::modelDefIDOpen()
{
    fileOpen(defidcard);
    if(defidcard.fileName().isEmpty())
  return;

    if(defidcard.libChosen())
  modelDefIDLoad();
    else
  updateView();
}


void EBTtool::modelDefIDNew()
{
    defidcard.setFileName(QString()); // is de basename!
    fileNew(defidcard.cFile);
    fileEdit(defidcard.cFile, FALSE);
}


void EBTtool::modelDefIDClean()
{
  if(defidcard.fileName().isEmpty()) return;
  defidcard.cleanDefIDcard();
  updateView();
  updateMenu(FALSE);
}


void EBTtool::modelDefOutEdit()
{
    defout.resetFileName();
    fileEdit(defout.cFile, FALSE);
}


void EBTtool::modelDefOutOpen()
{
    fileOpen(defout);
    if(defout.fileName().isEmpty())
  return;

    if(defout.libChosen())
  modelDefOutLoad();
    else
  updateView();
}


void EBTtool::modelDefOutNew()
{
    defout.setFileName(QString());  // is de basename!
    fileNew(defout.cFile);
    fileEdit(defout.cFile, FALSE);
}


void EBTtool::modelDefOutClean()
{
  if(defout.fileName().isEmpty()) return;
  defout.cleanDefOut();
  updateView();
  updateMenu(FALSE);
}


void EBTtool::modelHeaderEdit()
{
    model.resetFileName();
    fileEdit(model.hFile, FALSE);
}


void EBTtool::modelProgramEdit()
{
    model.resetFileName();
    fileEdit(model.cFile, FALSE);
}


void EBTtool::modelOpen()
{
    // onderstaande check ok?
    //    if (!ebtprFile.fileName().isEmpty() && !model.fileName().isEmpty())
    //    {
    //  bool containsModel = FALSE;
    //  EBTProject::EBTFileMap::Iterator i;
    //  for(i = currentProject.fileMap.begin(); i != currentProject.fileMap.end(); ++i)
    //      if(i.data().type == EBTMODEL)
    //      {
    //    containsModel = TRUE;
    //    break;
    //      }
    //  if(containsModel)
    //  {
    //      QMessageBox::warning(this, EBTCAPTION,
    //         "The current project already has a model assigned to it.\nIn order to be able to open another model, first close the \ncurrent project.");
    //      return;
    //  }
    //    }

    if((!model.hFile.inEditor()) == 0)
  if(!model.hFile.inEditor()->parentClose())
      return;
    if((!model.cFile.inEditor()) == 0)
  if(!model.cFile.inEditor()->parentClose())
      return;

    fileOpen(model);
    updateView();
}


void EBTtool::runDataClear()
{
    ClearDataForm dialog(this, TRUE);
    dialog.exec();
    getOutFileSize();
    //updateView();
}


void EBTtool::modelAllCopy()
{
  QString newname;

  if((!model.hFile.inEditor()) == 0)
    if(!model.hFile.inEditor()->parentClose())
      return;
  if((!model.cFile.inEditor()) == 0)
    if(!model.cFile.inEditor()->parentClose())
      return;
  if (QFile(model.hFile.absFilePath()).exists())
    newname = fileCopyAs(model.hFile, QString());

  if (QFile(model.cFile.absFilePath()).exists())
    {
      QFileInfo fi(newname);
      fileCopyAs(model.cFile, fi.absolutePath() + "/" + fi.completeBaseName() + ".c");
    }
}


void EBTtool::modelHeaderCopy()
{
  if((!model.hFile.inEditor()) == 0)
    if(!model.hFile.inEditor()->parentClose())
      return;
  if (QFile(model.hFile.absFilePath()).exists())
    fileCopyAs(model.hFile, QString());
}


void EBTtool::modelProgramCopy()
{
  if((!model.cFile.inEditor()) == 0)
    if(!model.cFile.inEditor()->parentClose())
      return;
  if (QFile(model.cFile.absFilePath()).exists())
    fileCopyAs(model.cFile, QString());
}


void EBTtool::runAllCopy()
{
  QString newname;

  if((!run.cvfFile.inEditor()) == 0)
    if(!run.cvfFile.inEditor()->parentClose())
      return;
  if((!run.isfFile.inEditor()) == 0)
    if(!run.isfFile.inEditor()->parentClose())
      return;

  if (QFile(run.cvfFile.absFilePath()).exists())
    newname = fileCopyAs(run.cvfFile, QString());
  if (QFile(run.isfFile.absFilePath()).exists())
    {
      QFileInfo fi(newname);
      fileCopyAs(run.isfFile, fi.absolutePath() + "/" + fi.completeBaseName() + "." + run.isfFile.fileExtension());
    }
}


void EBTtool::runCVFCopy()
{
  if((!run.cvfFile.inEditor()) == 0)
    if(!run.cvfFile.inEditor()->parentClose())
      return;
  if (QFile(run.cvfFile.absFilePath()).exists())
  fileCopyAs(run.cvfFile, QString());
}


void EBTtool::runISFCopy()
{
  if((!run.isfFile.inEditor()) == 0)
    if(!run.isfFile.inEditor()->parentClose())
      return;
  if (QFile(run.isfFile.absFilePath()).exists())
    fileCopyAs(run.isfFile, QString());
}


void EBTtool::runCVFEdit()
{
    run.resetFileName();
    fileEdit(run.cvfFile, FALSE);
}


void EBTtool::runISFEdit()
{
    run.resetFileName();
    fileEdit(run.isfFile, FALSE);
}

void EBTtool::runExecuteSlot()
{
  if((!outFile.inEditor()) == 0)
    if(!outFile.inEditor()->parentClose())
      return;
  outFile.setFileName(run.absFilePath() + "." + outFile.fileExtension());
  esfFile.setFileName(run.absFilePath() + "." + esfFile.fileExtension());
  repFile.setFileName(run.absFilePath() + "." + repFile.fileExtension());
  csbFile.setFileName(QString());

  if(!modelCompile(FALSE))
    return;

  oldDir = QDir::currentPath();
  QDir::setCurrent(run.dirPath());

  if(!modelInit())
    {
      QDir::setCurrent(oldDir);
      return;
    }

  outputRedraw = REDRAWPLOT;
  updateOutputGraphs();   // replottest
  updateView();

  if(errorMessage) delete errorMessage;
  errorMessage = new QErrorMessage(this);
  errorMessage->setModal(TRUE);

  updateMenu(TRUE);
  runStopAction->setEnabled(TRUE);
  runTogglePauseAction->setEnabled(TRUE);
  disableChildWidgets(TRUE);
  haveABreak = FALSE;

  connect(&thread, SIGNAL(runEnded()), 			    this,    SLOT(runEnded()));
  connect(&thread, SIGNAL(processErrors(QString, int)),     this,    SLOT(processErrors(QString, int)), Qt::BlockingQueuedConnection);
  connect(&thread, SIGNAL(processFPEerrors(QString, int )), this,    SLOT(processFPEerrors(QString, int )), Qt::BlockingQueuedConnection);
  connect(&thread, SIGNAL(updateStateGraphs()), 	    this,    SLOT(updateStateGraphs()), Qt::BlockingQueuedConnection);
  connect(&thread, SIGNAL(updateOutputImmediate()), 	    this,    SLOT(updateOutputGraphs()), Qt::BlockingQueuedConnection);
  connect(&thread, SIGNAL(updateOutputDelayed()), 	    this,    SLOT(updateOutputGraphs()));
  connect(this,    SIGNAL(setReady(bool)),		    &thread, SLOT(setReadyForSignal(bool)), Qt::DirectConnection);

  thread.newRun(&model, &outputData);
}

void EBTtool::runEnded()
{
  disconnect(&thread, SIGNAL(runEnded()), 		       0, 0);
  disconnect(&thread, SIGNAL(processErrors(QString, int)),     0, 0);
  disconnect(&thread, SIGNAL(processFPEerrors(QString, int )), 0, 0);
  disconnect(&thread, SIGNAL(updateStateGraphs()), 	       0, 0);
  disconnect(&thread, SIGNAL(updateOutputDelayed()), 	       0, 0);
  disconnect(this,    SIGNAL(setReady(bool)), 	       	       0, 0);

  QDir::setCurrent(oldDir);

  int retVal, setExcepts = 0;

  clearFPU();
  retVal = (model.shutDown(-1));
  setExcepts = testFPU();

  if (testExcepts(setExcepts))
    processFPEerrors("model.shutDown()", setExcepts);
  processErrors("model.shutDown()", retVal);

  for (int gg = 0; gg < ogList->size(); ++gg)
    updateOutputGraph(ogList->at(gg), REDRAWPLOT);

  //  if(graph->hasAutoScale()) updateOutputGraph(graph, REDRAWPLOT);

  runStopAction->setEnabled(FALSE);
  runTogglePauseAction->setEnabled(FALSE);
  updateView();
  updateMenu(FALSE);
  disableChildWidgets(FALSE);
  if (myArgv)
    {
      for(int ii = 0; ii < myArgc; ++ii)
	delete [] myArgv[ii];
      delete [] myArgv;
    }
  myArgc = 0;
  myArgv = NULL;
}

void EBTtool::runExtendSlot()
{
  runExtend = TRUE;
  runExecuteSlot();
  runExtend = FALSE;
}

void EBTtool::runTogglePause()
{
  if(haveABreak) //run was paused and is now resumed
    {
      haveABreak = FALSE;
      runTogglePauseAction->setText("Pause");
      runStopAction->setEnabled(TRUE);
      thread.resumeRun();
    }
  else
    {
      haveABreak = TRUE;
      thread.pauseRun();

      EBTGraphForm *graph;        //replottest
      for (int gg = 0; gg < ogList->size(); ++gg)
	{
	  graph = ogList->at(gg);
	  if(graph->hasAutoScale())
	    updateOutputGraph(graph, REDRAWPLOT);
	}
      runTogglePauseAction->setText("Resume");
      runExecuteAction->setEnabled(FALSE);
      runStopAction->setEnabled(FALSE);
      disableChildWidgets(FALSE);
    }
}

void EBTtool::runStop()
{
    thread.quitRun();
}

void EBTtool::runAllNew()
{
    if((!run.cvfFile.inEditor()) == 0)
  if(!run.cvfFile.inEditor()->parentClose())
      return;
    if((!run.isfFile.inEditor()) == 0)
  if(!run.isfFile.inEditor()->parentClose())
      return;

    runCVFNew();
    runISFNew();
    fileSaveAs(run);
    run.cvfFile.inEditor()->updateView();
    run.isfFile.inEditor()->updateView();
    updateView();
    updateMenu(FALSE);
}

void EBTtool::runCVFNew()
{
    if((!run.cvfFile.inEditor()) == 0)
  if(!run.cvfFile.inEditor()->parentClose())
      return;
    fileNew(run.cvfFile);
    fileEdit(run.cvfFile, FALSE);
}

void EBTtool::runISFNew()
{
    if((!run.isfFile.inEditor()) == 0)
  if(!run.isfFile.inEditor()->parentClose())
      return;
    fileNew(run.isfFile);
    fileEdit(run.isfFile, FALSE);
}

void EBTtool::runOpen()
{
    if((!run.cvfFile.inEditor()) == 0)
  if(!run.cvfFile.inEditor()->parentClose())
      return;
    if((!run.isfFile.inEditor()) == 0)
  if(!run.isfFile.inEditor()->parentClose())
      return;

    fileOpen(run);

    if(!run.fileName().isEmpty())
    {
  if((!outFile.inEditor()) == 0)
      if(!outFile.inEditor()->parentClose())
    return;  //nog geen perfecte oplossing...

  outFile.setFileName(run.absFilePath() + "." + outFile.fileExtension());
  esfFile.setFileName(run.absFilePath() + "." + esfFile.fileExtension());
  repFile.setFileName(run.absFilePath() + "." + repFile.fileExtension());
  csbFile.setFileName(QString());
    }
    updateView();
    updateMenu(FALSE);
}

void EBTtool::fileStateOutputOpen()
{
  if (errorMessage) delete errorMessage;
  errorMessage = new QErrorMessage(this);
  errorMessage->setModal(TRUE);

  fileOpen(csbFile);
  if (!csbFile.fileName().isEmpty()) csbFile.scan();

  currentStateRecord = 0;
  if (defidcard.libLoaded()) 
    {
      csbFile.clearIDcard();
      csbFile.setDefID(&defidcard);
    }

  if (csbFile.scanSucceeded()) stateOutputBrowse(REDRAWPLOT);
  stateBrowseScrollBar->setMinimum(1);
  stateBrowseScrollBar->setMaximum(csbFile.numStates());
  //    defout.libUnload();
  //    if(!csbFile.fileName().isEmpty())
  //  csbFile.setDefOut(&defout);
  if (defout.libLoaded())
    {
      bool ok = TRUE;

      if (computedData.count() > 0)
        {
          ok = (QMessageBox::question(this, EBTCAPTION + "Clear computed", QString("Are you sure you want to delete\nthe current computed data?"),
                                      QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes);
          if (ok) computedData.clear();
        }

      if (ok)
        {
          csbFile.setDefOut(&defout);                                               // OK??
          fillComputedData();
        }
    }
  updateView();
  updateMenu(FALSE);
}

void EBTtool::updateView()
{
    QFileInfo fi;
    QString displayString;

    if(!model.fileName().isEmpty())
    {
  fi.setFile(model.absFilePath());
  textDisplayModel->setText(fi.baseName());
    }
    else
  textDisplayModel->setText(QString());
    if(!run.fileName().isEmpty())
    {
  fi.setFile(run.absFilePath());
  textDisplayRun->setText(fi.baseName());
    }
    else
  textDisplayRun->setText(QString());
    textDisplayArgs->setText(runArgs);

    if (!outFile.fileName().isEmpty())
    {
  displayString = outFile.fileName() + QString(" [%1 columns, %2 rows]")
      .arg((int)outFile.numColumns())
    .arg((int)outFile.numRows());
    }
    else
  displayString = QString();
    textDisplayData->setText(displayString);

    if (!csbFile.fileName().isEmpty())
    {
  if(csbFile.scanSucceeded())
  {
      QString pluralP = "";
      QString pluralC = "";
      if(csbFile.numPopulations(currentStateRecord) != 1)
    pluralP = "s";
      if(csbFile.numColumns(currentStateRecord, 0) != 1)
    pluralC = "s";
      displayString = csbFile.fileName()
          + QString("\n[record %1 of %2: %3 population%4, population 0 has %5 column%6 of length %7]")
          .arg(currentStateRecord + 1)
          .arg((int)csbFile.numStates())
          .arg((int)csbFile.numPopulations(currentStateRecord))
          .arg(pluralP)
          .arg((int)csbFile.numColumns(currentStateRecord, 0))
          .arg(pluralC)
          .arg((int)csbFile.numCohorts(currentStateRecord, 0));
  }
  else
      displayString = csbFile.fileName() + QString(" (scan was unsuccessful)");
    }
    else
  displayString = QString();
    textDisplayState->setText(displayString);

    if(!ebtprFile.fileName().isEmpty() && !currentProject.saveTime().isEmpty())
  textDisplayProject->setText(ebtprFile.fileName() + " (" + currentProject.saveTime() + ")");
    else
  textDisplayProject->setText(ebtprFile.fileName());

    if(!defidcard.fileName().isEmpty() && defidcard.libLoaded())
  textDisplayDefID->setText(defidcard.fileName() + " [loaded]");
    else
  textDisplayDefID->setText(defidcard.fileName());

    if(!defout.fileName().isEmpty() && defout.libLoaded())
  textDisplayDefOut->setText(defout.fileName() + " [loaded]");
    else
  textDisplayDefOut->setText(defout.fileName());

    //    updateMenu(FALSE);
}

void EBTtool::projectSave()
{
  currentProject.clearMap();
  updateProject();
  ebtprFile.project = currentProject;
  if (ebtprFile.save())
    {
      setWindowTitle(ebtprFile.captionText());
      statusBar()->showMessage(QString("Saved project to '%1'").
	  arg(ebtprFile.fileName()), 3000);
      ebtprFile.setChanged(FALSE);
    }
  else
    QMessageBox::warning(this, QString("Warning"), QString("Failed to save '%1'").arg(ebtprFile.fileName()));

  updateView(); // to show correct saveTime
}

void EBTtool::projectSaveAs()
{
    if(ebtprFile.fileName().isEmpty())
  fileNew(ebtprFile);
    projectEdit();
    currentProject.clearMap();
    updateProject();
    ebtprFile.project = currentProject;
    fileSaveAs(ebtprFile);
    updateView();
}

bool EBTtool::okToClear()
{
  TextEditor *te;
  while(teList->size() > 0)
    {
      te = teList->first();
      if(!te->parentClose()) return FALSE;
    }
  EBTGraphForm *graph;
  while(ogList->size() > 0)
    {
      graph = ogList->first();
      if(!graph->parentClose()) return FALSE;
    }
  while(sgList->size() > 0)
    {
      graph = sgList->first();
      if(!graph->parentClose()) return FALSE;
    }

  if(!ebtprFile.okToClear()) return FALSE;

  return TRUE;
}

void EBTtool::closeEvent(QCloseEvent *)
{
  fileExit();
}

void EBTtool::fileEdit(EBTFile& ef, bool viewOnly)
{
  if((!ef.inEditor()) == 0)
    if(!ef.inEditor()->parentClose()) return;

  if (!ef.fileName().isEmpty() || ef.isNew())
    {
      TextEditor *editor = new TextEditor(this);
      teList->append(editor);
      editor->setFile(ef, viewOnly);
      editor->show();
      ef.setEditor(editor);
    }
  else
    statusBar()->showMessage("No such file: empty file name", 3000);
}

void EBTtool::projectNew()
{
    //    fileNew(ebtprFile);
    //    loadProject();
    //    fileProjectEdit();
}

void EBTtool::modelNew()
{
    if((!model.hFile.inEditor()) == 0)
  if(!model.hFile.inEditor()->parentClose())
      return;
    if((!model.cFile.inEditor()) == 0)
  if(!model.cFile.inEditor()->parentClose())
      return;
    model.setFileName(QString());
    fileNew(model.hFile);
    fileNew(model.cFile);
    fileEdit(model.hFile, FALSE);
    fileEdit(model.cFile, FALSE);
    fileSaveAs(model);
    model.cFile.inEditor()->updateView();
    model.hFile.inEditor()->updateView();
    updateView();
    updateMenu(FALSE);
}

bool EBTtool::modelInit()
{
  if(!model.initHandles()) return FALSE;

  if (myArgv)
    {
      for(int ii = 0; ii < myArgc; ++ii)
	delete [] myArgv[ii];
      delete [] myArgv;
    }
  myArgc = 0;
  myArgv = NULL;

  QStringList runArgList = runArgs.split(" ", Qt::SkipEmptyParts);
  if (runExtend) runArgList.append("-r");
  runArgList.append(NULL);
  myArgc = 2 + runArgList.count();  // model name, run name, runtime arguments

  // c_model already deleted in the myArgv loop above
  c_model = new char[model.absFilePath().length() + 1];
  STRCPY(c_model, model.absFilePath().toLatin1().constData ());				// Qt4

  // c_run already deleted in the myArgv loop above
  c_run = new char[run.absFilePath().length() + 1];
  STRCPY(c_run, run.absFilePath().toLatin1().constData ());				// Qt4

  myArgv = new charPtr[myArgc];
  myArgv[0] = c_model;
  myArgv[1] = c_run;

  for (int runArgNr = 0; runArgNr < runArgList.count(); runArgNr++)
    {
      if(!runArgList[runArgNr].isNull())
	{
	  // charRunArg already deleted in the myArgv loop above
	  charRunArg = new char[runArgList[runArgNr].length() + 1];
	  STRCPY(charRunArg, runArgList[runArgNr].toLatin1().constData ());		// Qt4
	}
      else
	charRunArg = NULL;

      myArgv[2+runArgNr] = charRunArg;
    }

  int retVal, setExcepts = 0;


#ifdef DEBUG
  qDebug() <<  "EBTtool::modelInit: calling model.startUp()";
#endif
  clearFPU();
  retVal = (model.startUp(myArgc, myArgv));
  setExcepts = testFPU();

  if (testExcepts(setExcepts))
    {
      processFPEerrors("model.startUp()", setExcepts);
      runEnded();
      return FALSE;
    }
  else processErrors("model.startUp()", retVal);

  if (retVal & FATAL_ERROR)
    {
      runEnded();
      return FALSE;
    }

  // output data initialisation
  EBTGraphForm *graph;
  for (int gg = 0; gg < ogList->size(); ++gg)
    {
      graph = ogList->at(gg);
      for (uint ii = 0; ii < graph->currentSettings.dataSetMapSize(); ii++)
	graph->currentSettings.clearDataSet(ii);
    }

  outputData.clear();
  outputRedraw = REDRAWPLOT;
  stateRedraw  = REDRAWPLOT;

  for(uint colNr = 0; colNr < model.outputVarNum() + 1; colNr++)
    {
      EBTData dataVec;
      outputData[colNr] = dataVec;
    }

  if (retVal & NORMAL_OUT)
    {
      for (uint colNr = 0; colNr < model.outputVarNum() + 1; colNr++)
	outputData[colNr].appendValue(model.outputData()[colNr]);
      updateOutputGraphs();
    }
  if (retVal & STATE_OUT) updateStateGraphs();

  return TRUE;
} //EBTtool::modelInit()


void EBTtool::loadOutputValues(uint appendData)
{
  statusBar()->showMessage(QString("Loading '%1'...").arg(outFile.fileName()));
  statusBar()->repaint();
  if (appendData == APPENDDATA)
    {
      outputData = outFile.appendData(outputData);
    }
  else
    {
      outputData.clear();
      outputData = outFile.loadData();
    }

  updateView();
  updateMenu(FALSE);

  for (int gg = 0; gg < ogList->size(); ++gg)
    updateOutputGraph(ogList->at(gg), REDRAWPLOT);
  statusBar()->showMessage("Ready");
}

void EBTtool::loadComputedValues(uint appendData)
{
    statusBar()->showMessage(QString("Loading '%1'...").arg(compFile.fileName()));
    statusBar()->repaint();
    if (appendData == APPENDDATA)
      {
  computedData = compFile.appendData(computedData);
      }
    else
      {
  computedData.clear();
  computedData = compFile.loadData();
      }

    updateView();
    updateMenu(FALSE);

    for (int gg = 0; gg < ogList->size(); ++gg)
      updateOutputGraph(ogList->at(gg), REDRAWPLOT);
    statusBar()->showMessage("Ready");
}

void EBTtool::loadFixedValues(uint appendData)
{
    statusBar()->showMessage(QString("Loading '%1'...").arg(fixFile.fileName()));
    statusBar()->repaint();
    if (appendData == APPENDDATA)
      {
  fixedData = fixFile.appendData(fixedData);
      }
    else
      {
  fixedData.clear();
  fixedData = fixFile.loadData();
      }

    updateView();
    updateMenu(FALSE);

    for (int gg = 0; gg < ogList->size(); ++gg)
      updateOutputGraph(ogList->at(gg), REDRAWPLOT);
    statusBar()->showMessage("Ready");
}

void EBTtool::updateStateGraphs()
{
  EBTGraphForm *graph;
  uint          xCol, yCol, popNr = 0, ii, cohortNr;
  int           gg;
  bool          colError = FALSE;
  bool          popError = FALSE;
  double        xVal, yVal;

  for (gg = 0; gg < sgList->size(); ++gg)
    {
      graph = sgList->at(gg);

      for (ii = 0; ii < graph->currentSettings.dataSetMapSize(); ii++)
        {
          graph->currentSettings.clearDataSet(ii);
          popNr = graph->currentSettings.dataSetPopulationNr(ii);
          xCol  = graph->currentSettings.dataSetXCol(ii) - 1;
          yCol  = graph->currentSettings.dataSetYCol(ii) - 1;
          if (xCol > (model.iStateDim() + model.iConstDim()) || yCol > (model.iStateDim() + model.iConstDim()) || popNr >= model.populationNum())
            {
              if (xCol > (model.iStateDim() + model.iConstDim()) || yCol > (model.iStateDim() + model.iConstDim())) colError = TRUE;
              if (popNr >= model.populationNum()) popError = TRUE;
              continue;
            }

          for (cohortNr = 0; cohortNr < (uint)model.cohortNum()[popNr]; cohortNr++)
            {
              if (xCol <= model.iStateDim())
                xVal = *(((double *)model.population()[popNr]) + cohortNr * (1 + model.iStateDim()) + xCol);
              else
                xVal = *(((double *)model.popIDcard()[popNr]) + cohortNr * (model.iConstDim()) + (xCol - model.iStateDim() - 1));
              if (yCol <= model.iStateDim())
                yVal = *(((double *)model.population()[popNr]) + cohortNr * (1 + model.iStateDim()) + yCol);
              else
                yVal = *(((double *)model.popIDcard()[popNr]) + cohortNr * (model.iConstDim()) + (yCol - model.iStateDim() - 1));
              graph->currentSettings.dataSetXAppend(ii, xVal);
              graph->currentSettings.dataSetYAppend(ii, yVal);
            }
        }

      if (colError)
        {
	        errorMessage->showMessage(colWarning);
          colError = FALSE;
        }
      if (popError)
        {
	        errorMessage->showMessage(popWarning);
          popError = FALSE;
        }
      if (!model.stateLabel(popNr).isEmpty())
        graph->showText(model.stateLabel(popNr));
      else
        graph->showText(QString("State graph at t=%1").arg(formatDouble((double)model.environment()[0])));

      updateGraph(graph, stateRedraw);
    }                                                                                // for each state graph

  stateRedraw = REDRAWCURVES;
}

void EBTtool::updateOutputGraphs()
{
  EBTGraphForm *graph;
  uint          xCol, yCol;
  uint          ii, jj;
  int           gg;
  static int    done = 1;
  double        xVal, yVal, xAxisMin, xAxisMax, yAxisMin, yAxisMax;
  bool          incPLot;

  if (!done) return;
  done         = 0;
  uint oldSize = outputData[0].getSizeRead();
  uint newSize = outputData[0].size();

  if ((outputRedraw == DRAWPOINTS) && (oldSize >= newSize))
    {
      outputData[0].setSizeRead(newSize);
      done = 1;
      return;
    }

  for (gg = 0; gg < ogList->size(); ++gg)
    {
      graph = ogList->at(gg);

      incPLot = (outputRedraw == DRAWPOINTS);
      for (ii = 0; ii < graph->currentSettings.dataSetMapSize(); ii++)
        {
          if (graph->currentSettings.dataSetDataType(ii) != OUTPUTVALUES) continue;

          xCol = graph->currentSettings.dataSetXCol(ii) - 1;
          yCol = graph->currentSettings.dataSetYCol(ii) - 1;
          if (xCol > model.outputVarNum() || yCol > model.outputVarNum()) continue;

          xAxisMin = graph->axisMinX(ii);
          xAxisMax = graph->axisMaxX(ii);
          yAxisMin = -0.1 * DBL_MAX;
          yAxisMax = 0.1 * DBL_MAX;
          if (graph->currentSettings.axisLogarithmic(graph->currentSettings.dataSetYAxis(ii))) yAxisMin = MIN_LOG_VAL;

          // Only do an incremental plot if data sets already contained points before
          incPLot &= (graph->currentSettings.dataSetSize(ii) > 0);

          for (jj = oldSize; jj < newSize; jj++)
            {
              xVal = outputData[xCol].dataPnt()[jj];
              yVal = outputData[yCol].dataPnt()[jj];

              bool xOk = (xVal >= xAxisMin) && (xVal <= xAxisMax);
              bool yOk = (yVal >= yAxisMin) && (yVal <= yAxisMax);

              if (xOk && yOk)
                {
                  graph->currentSettings.dataSetXAppend(ii, xVal);
                  graph->currentSettings.dataSetYAppend(ii, yVal);
                  graph->setDirtyFlag(TRUE);
                }
              else if (xOk)
                {
                  uint lastsize = graph->currentSettings.dataSetSize(ii);
                  if (lastsize && isnan((graph->currentSettings.dataSetYData(ii))[lastsize - 1])) continue;

                  graph->currentSettings.dataSetXAppend(ii, xVal);
                  graph->currentSettings.dataSetYAppend(ii, NAN);
                  graph->setDirtyFlag(TRUE);
                }
            }
        }                                                                            // for all datasets

#ifdef DEBUG
      qDebug() << "updateOutputGraphs: Added " << newSize - oldSize << "data points to graph" << gg;
#endif
      if ((outputRedraw == REDRAWPLOT) || graph->getDirtyFlag())
        {
          if (incPLot)
            updateGraph(graph, outputRedraw);
          else
            updateGraph(graph, REDRAWPLOT);
          graph->setDirtyFlag(FALSE);
        }
    }                                                                                // for each output graph

  outputData[0].setSizeRead(newSize);

  // Only allow for incremental data plotting after some data have been plotted already. Otherwise crash occur
  outputRedraw = DRAWPOINTS;
  done         = 1;
  emit setReady(true);

  return;
}


void EBTtool::updateGraph( EBTGraphForm *graph, int redrawType )
{
  switch(redrawType)
    {
    case REDRAWPLOT:
      graph->applySettings();
      break;
    case REDRAWCURVES:				// This is only executed for state graphs
      graph->redrawData();
      break;
    case DRAWPOINTS:
      graph->incrementalOutputPlot();
      break;
    default:
      graph->applySettings();
    }
}

void EBTtool::loadProject()
{
  statusBar()->showMessage(QString("Loading '%1'...").arg(ebtprFile.fileName()));
  ebtprFile.load();
  currentProject = ebtprFile.project;
  setWindowTitle(ebtprFile.captionText());

  EBTOGrFile *ebtogrFile;
  EBTSGrFile *ebtsgrFile;
  for(uint ii = 0; ii < currentProject.fileMapSize(); ii++)
    {
      switch(currentProject.fileRefType(ii))
	{
	case EBTMODEL:
	  model.setFileName(currentProject.fileRefName(ii));  // is de basename!
	  break;
	case EBTRUN:
	  if((!run.cvfFile.inEditor()) == 0)
	    if(!run.cvfFile.inEditor()->parentClose())
	      break;
	  if((!run.isfFile.inEditor()) == 0)
	    if(!run.isfFile.inEditor()->parentClose())
	      break;
	  run.setFileName(currentProject.fileRefName(ii));  // is de basename!

	  if(!run.fileName().isEmpty())
	    {
	      if((!outFile.inEditor()) == 0)
		if(!outFile.inEditor()->parentClose())
		  break;  //nog geen perfecte oplossing...

	      outFile.setFileName(run.absFilePath() + "." + outFile.fileExtension());
	      esfFile.setFileName(run.absFilePath() + "." + esfFile.fileExtension());
	      repFile.setFileName(run.absFilePath() + "." + repFile.fileExtension());
	      csbFile.setFileName(QString());
	    }
	  break;
	case EBTDEFID:
	  defidcard.setFileName(currentProject.fileRefName(ii));
	  break;
	case EBTDEFOUT:
	  defout.setFileName(currentProject.fileRefName(ii));
	  break;
	case EBTOGR:
	  ebtogrFile = new EBTOGrFile;
	  ebtogrFile->setFileName(currentProject.fileRefName(ii));
	  outputGraphDirPath = ebtogrFile->dirPath();
	  graphOutputGraph(*ebtogrFile, currentProject.fileRefXPos(ii), currentProject.fileRefYPos(ii),
	      currentProject.fileRefWidth(ii), currentProject.fileRefHeight(ii));
	  break;
	case EBTSGR:
	  ebtsgrFile = new EBTSGrFile;
	  ebtsgrFile->setFileName(currentProject.fileRefName(ii));
	  stateGraphDirPath = ebtsgrFile->dirPath();
	  graphStateGraph(*ebtsgrFile, currentProject.fileRefXPos(ii), currentProject.fileRefYPos(ii),
	      currentProject.fileRefWidth(ii), currentProject.fileRefHeight(ii));
	  break;
	default:
	  break;
	}
    }
  runArgs = currentProject.runtimeArguments();

  ebtprFile.setChanged(FALSE);
  updateView();
  updateMenu(FALSE);
  statusBar()->showMessage("Ready");
}

void EBTtool::projectOpen()
{
  EBTPrFile tmpprFile;

  if (!(ebtprFile.fileName().isEmpty() || ebtprFile.fileName().isNull()))
    QDir::setCurrent(ebtprFile.dirPath());
  fileOpen(tmpprFile);
  if(tmpprFile.fileName().isEmpty()) return;

  projectClose();
  ebtprFile = tmpprFile;
  loadProject();
  updateView();
}

void EBTtool::runCVFView()
{
    run.resetFileName();
    fileEdit(run.cvfFile, TRUE);
}

void EBTtool::runISFView()
{
    run.resetFileName();
    fileEdit(run.isfFile, TRUE);
}

void EBTtool::runESFView()
{
    fileEdit(esfFile, TRUE);
}

void EBTtool::runESFCopyToISF()
{
  if((!esfFile.inEditor()) == 0)
    if(!esfFile.inEditor()->parentClose())
      return;

  if (QFile(esfFile.absFilePath()).exists())
    {
      QString isfName;

      isfName = QFileDialog::getSaveFileName(this,
	  tr("Copy ESF to ISF"), 0, tr("ISF files (*.isf)"));
      if (isfName.isEmpty()) return;

      QFileInfo isfFileInfo(isfName);
      isfName = isfFileInfo.absolutePath() + "/" + isfFileInfo.completeBaseName() + ".isf";

      if (QFile(isfName).exists()) QFile(isfName).remove();

// 	Got the full name of the new ISF file now; straight copy ESF to ISF
//      QFile(esfFile.absFilePath()).copy(isfName);

      QFile file(esfFile.absFilePath());
      QString outputString;
      bool first = TRUE;
      QString str, tmpStr;
      QStringList strList;

      if (file.open(QIODevice::ReadOnly | QIODevice::Text))
	{
	  QTextStream stream(&file);
	  while ((str = stream.readLine()) != QString())
	    {
	      if ((str.length() > 1) && (!str.startsWith("#")) && (!str.startsWith("@")))
		{
		  tmpStr = str.replace(QChar('\n'), " ").replace(QChar('\t'), " ");
		  strList = tmpStr.split(" ", Qt::SkipEmptyParts);

		  int nColsRead = strList.count();
		  for (int colNum = 0; colNum < nColsRead; colNum++)
		    {
		      if(colNum > 0) outputString += "\t";
		      double value = strList[colNum].toDouble(); // toDouble() will return 0.0 when conversion fails
		      if ((first && !colNum) || (fabs(value) < ZERO_VAL)) outputString += formatDouble(0.0);
		      else outputString += strList[colNum];
		    }
		  first = FALSE;
		} // if(str.length() > 1)
	      else outputString += str;
	      outputString += "\n";
	    } // while readline
	}
      file.close();

      QFile isfFile(isfName);

      if (isfFile.open(QIODevice::WriteOnly))
	{
	  QTextStream stream(&isfFile);
          stream << outputString;
          isfFile.close();
	}
    }

  return;
}

void EBTtool::runRepView()
{
    fileEdit(repFile, TRUE);
}

void EBTtool::runOutputValuesView()
{
  if (!outFile.fileName().isEmpty())
    {
      TextEditor *editor = new TextEditor(this);
      teList->append(editor);
      editor->setFile(outFile, TRUE);
      editor->show();
   }
  else
    statusBar()->showMessage("No such file open", 3000);
}

void EBTtool::initFiles()
{
    outFile.setFileExtension("out");
    outFile.setDescriptionText("Output values");
    outFile.setOpenText(outFile.descriptionText() + " (*." + outFile.fileExtension() + ");;Data files (*.dat);;Text files (*.txt);;Any files (*)");

    compFile.setFileExtension("out");
    compFile.setDescriptionText("Computed output values");
    compFile.setOpenText(compFile.descriptionText() + " (*." + compFile.fileExtension() + ");;Data files (*.dat);;Text files (*.txt);;Any files (*)");

    fixFile.setFileExtension("out");
    fixFile.setDescriptionText("Fixed output values");
    fixFile.setOpenText(fixFile.descriptionText() + " (*." + fixFile.fileExtension() + ");;Data files (*.dat);;Text files (*.txt);;Any files (*)");

    csbFile.setFileExtension("csb");
    csbFile.setDescriptionText("State output");
    csbFile.setOpenText(csbFile.descriptionText() + " (*." + csbFile.fileExtension() + ")") ;

    esfFile.setFileExtension("esf");
    esfFile.setDescriptionText("End state file");
    esfFile.setOpenText(esfFile.descriptionText() + " (*." + esfFile.fileExtension() + ")") ;

    repFile.setFileExtension("rep");
    repFile.setDescriptionText("Report file");
    repFile.setOpenText(repFile.descriptionText() + " (*." + repFile.fileExtension() + ")") ;

    ebtprFile.setFileExtension("ebtpr");
    ebtprFile.setDescriptionText("EBT project");
    ebtprFile.setSaveText(ebtprFile.descriptionText() + " (*." + ebtprFile.fileExtension() + ")") ;
    ebtprFile.setOpenText(ebtprFile.saveText());
}

void EBTtool::updateMenu(bool allDisabled)
{
    bool runDisabled = run.fileName().isNull();
    bool modelDisabled = model.fileName().isNull();

    fileOutputValuesOpenAction->setDisabled(allDisabled);
    fileComputedValuesOpenAction->setDisabled(allDisabled);
    fileFixedValuesOpenAction->setDisabled(allDisabled);
    fileStateOutputOpenAction->setDisabled(allDisabled);
    fileRereadAction->setDisabled(allDisabled || outFile.fileName().isNull());

    buttonRefresh->setHidden(outFile.fileName().isNull());
    buttonRefresh->setDisabled(allDisabled /* || outFile.fileName().isNull()*/ );

    hideCSBControls(csbFile.fileName().isNull() || !csbFile.scanSucceeded());
    disableCSBControls(allDisabled);

    statePause->setDisabled(allDisabled || !timer->isActive());
    stateBack->setDisabled(allDisabled || (currentStateRecord == 0));
    stateForward->setDisabled(allDisabled || (currentStateRecord == ((int)csbFile.numStates() - 1)));
    stateBrowseScrollBar->setValue(currentStateRecord + 1);
    stateExportAction->setDisabled(allDisabled || (stateBrowseScrollBar->isHidden() && !model.libLoaded()));

    //    fileProjectNewAction->setDisabled(allDisabled);
    projectOpenAction->setDisabled(allDisabled);
    projectCloseAction->setDisabled(allDisabled || ebtprFile.fileName().isNull());
    projectEditAction->setDisabled(allDisabled || ebtprFile.fileName().isNull());
    projectSaveAction->setDisabled(allDisabled || ebtprFile.fileName().isNull());
    projectSaveAsAction->setDisabled(allDisabled || (modelDisabled && runDisabled && sgList->isEmpty() && ogList->isEmpty()));

    fileExitAction->setDisabled(allDisabled);

    modelOpenAction->setDisabled(allDisabled);

    modelHeaderEditAction->setDisabled(allDisabled || modelDisabled);
    modelProgramEditAction->setDisabled(allDisabled || modelDisabled);

    modelAllCopyAction->setDisabled(allDisabled || modelDisabled);
    modelHeaderCopyAction->setDisabled(allDisabled || modelDisabled);
    modelProgramCopyAction->setDisabled(allDisabled || modelDisabled);

    modelNewAction->setDisabled(allDisabled);

    modelCompileAction->setDisabled(allDisabled || modelDisabled);

    modelDefOutOpenAction->setDisabled(allDisabled);
    if(defout.libLoaded())
    {
  modelDefOutLoadAction->setText("Unload");
  modelDefOutLoadAction->setToolTip("Unload defoutput");
  modelDefOutLoadAction->setChecked(TRUE);
    }
    else
    {
  modelDefOutLoadAction->setText("Load");
  modelDefOutLoadAction->setToolTip("Load defoutput");
  modelDefOutLoadAction->setChecked(FALSE);
    }
    modelDefOutLoadAction->setDisabled(allDisabled || defout.fileName().isNull());
    modelDefOutEditAction->setDisabled(allDisabled || defout.fileName().isNull());
    modelDefOutNewAction->setDisabled(allDisabled);
    modelDefOutCleanAction->setDisabled(allDisabled || defout.fileName().isNull());
    fileComputedValuesSaveAction->setDisabled(allDisabled
                || stateBrowseScrollBar->isHidden()
                || !defout.libLoaded());
    modelDefIDOpenAction->setDisabled(allDisabled);
    if(defidcard.libLoaded())
    {
  modelDefIDLoadAction->setText("Unload");
  modelDefIDLoadAction->setToolTip("Unload defidcard");
  modelDefIDLoadAction->setChecked(TRUE);
    }
    else
    {
  modelDefIDLoadAction->setText("Load");
  modelDefIDLoadAction->setToolTip("Load defidcard");
  modelDefIDLoadAction->setChecked(FALSE);
    }
    modelDefIDLoadAction->setDisabled(allDisabled || defidcard.fileName().isNull());
    modelDefIDEditAction->setDisabled(allDisabled || defidcard.fileName().isNull());
    modelDefIDNewAction->setDisabled(allDisabled);
    modelDefIDCleanAction->setDisabled(allDisabled || defidcard.fileName().isNull());

    runOpenAction->setDisabled(allDisabled);

    runCVFViewAction->setDisabled(allDisabled || runDisabled);
    runISFViewAction->setDisabled(allDisabled || runDisabled ||
                                  run.isfFile.fileName().isEmpty() ||
                                  !QFile(run.isfFile.absFilePath()).exists());

    runOutputValuesViewAction->setDisabled(allDisabled || runDisabled ||
                                  outFile.fileName().isEmpty() ||
                                  !QFile(outFile.absFilePath()).exists());
    runESFViewAction->setDisabled(allDisabled || runDisabled ||
                                  esfFile.fileName().isEmpty() ||
                                  !QFile(esfFile.absFilePath()).exists());
    runRepViewAction->setDisabled(allDisabled || runDisabled ||
                                  repFile.fileName().isEmpty() ||
                                  !QFile(repFile.absFilePath()).exists());

    runCVFEditAction->setDisabled(allDisabled || runDisabled);
    runISFEditAction->setDisabled(allDisabled || runDisabled ||
                                  run.isfFile.fileName().isEmpty() ||
                                  !QFile(run.isfFile.absFilePath()).exists());

    runCVFCopyAction->setDisabled(allDisabled || runDisabled);
    runISFCopyAction->setDisabled(allDisabled || runDisabled ||
                                  run.isfFile.fileName().isEmpty() ||
                                  !QFile(run.isfFile.absFilePath()).exists());
    runAllCopyAction->setDisabled(allDisabled || runDisabled ||
                                  run.isfFile.fileName().isEmpty() ||
                                  !QFile(run.isfFile.absFilePath()).exists());
    runESFCopyAction->setDisabled(allDisabled || runDisabled ||
                                  esfFile.fileName().isEmpty() ||
                                  !QFile(esfFile.absFilePath()).exists());

#ifdef DEBUG
    qDebug() << "File:" << esfFile.fileName() << "Empty:" << esfFile.fileName().isEmpty() << "Exists:" << QFile(esfFile.absFilePath()).exists();
#endif

    runCVFNewAction->setDisabled(allDisabled);
    runISFNewAction->setDisabled(allDisabled);
    runAllNewAction->setDisabled(allDisabled);

    runArgumentsEditAction->setDisabled(allDisabled || runDisabled);

    runExecuteAction->setDisabled(allDisabled || runDisabled || modelDisabled);
    runExtendAction->setDisabled(allDisabled || runDisabled || modelDisabled ||
                                  esfFile.fileName().isEmpty() ||
                                  !QFile(esfFile.absFilePath()).exists());

    runDataClearAction->setDisabled(allDisabled
            || (outFile.fileName().isEmpty()
          && compFile.fileName().isEmpty()
          && fixFile.fileName().isEmpty()
          && csbFile.fileName().isEmpty()
          && outputData.count() == 0
          && computedData.count() == 0
          && fixedData.count() == 0
          ));

    graphOutputGraphOpenAction->setDisabled(allDisabled);
    graphStateGraphOpenAction->setDisabled(allDisabled);
    graphOutputGraphNewAction->setDisabled(allDisabled);
    graphStateGraphNewAction->setDisabled(allDisabled);

    helpContentsAction->setDisabled(allDisabled);
    helpIndexAction->setDisabled(allDisabled);
    helpAboutAction->setDisabled(allDisabled);
    helpAboutQtAction->setDisabled(allDisabled);
}

void EBTtool::updateGraphMenu()
{
  EBTGraphForm *graph;
  QAction *newAction;

  Graph->clear();
  Graph->setTitle(QApplication::translate("EBTtool", "&Graph", 0));
  Graph->addAction(graphOpenSubMenu->menuAction());
  Graph->addAction(graphNewSubMenu->menuAction());

  Graph->addSeparator();
  for (int gg = 0; gg < ogList->size(); ++gg)
    {
      graph = ogList->at(gg);
      if (!graph->getHideFlag())
  newAction = Graph->addAction(QIcon(":/images/check_checked.png"),
             graph->windowTitle().remove(0, 11), graph, SLOT(graphShowAndRaiseSlot(void)));
      else
  newAction = Graph->addAction(graph->windowTitle().remove(0, 11), graph, SLOT(graphShowAndRaiseSlot(void)));
      graph->setGraphMenuItem(newAction);
    }

  Graph->addSeparator();
  for (int gg = 0; gg < sgList->size(); ++gg)
    {
      graph = sgList->at(gg);
      if (!graph->getHideFlag())
  newAction = Graph->addAction(QIcon(":/images/check_checked.png"),
             graph->windowTitle().remove(0, 11), graph, SLOT(graphShowAndRaiseSlot(void)));
      else
  newAction = Graph->addAction(graph->windowTitle().remove(0, 11), graph, SLOT(graphShowAndRaiseSlot(void)));
      graph->setGraphMenuItem(newAction);
    }
}

void EBTtool::runArgumentsEdit()
{
    bool ok;
    QString argsText = QInputDialog::getText(this, EBTCAPTION + "arguments", "Runtime arguments:",
					     QLineEdit::Normal, runArgs, &ok);
    if ( ok )
    {
  runArgs = argsText;
  updateView();
    }
    // otherwise, keep old runArgs
}

void EBTtool::updateGraphSlot( EBTGraphForm *graph, bool isOutputGraph)
{
  if(isOutputGraph)
    updateOutputGraph(graph, REDRAWPLOT);
  else
    stateOutputBrowse(REDRAWPLOT);

#ifdef DEBUG
    qDebug("updateGraphSlot() called");
#endif
}

void EBTtool::updateOutputGraph(EBTGraphForm *graph, uint redrawType)
{
  int xCol, yCol;
  EBTDataMap theData;

  for (uint ii = 0; ii < graph->currentSettings.dataSetMapSize(); ii++)
    {
      // since QMap is an implicitly shared class (see Qt ref. doc.) the assignments
      // to theData, below, will not cause extra memory to be used
      switch(graph->currentSettings.dataSetDataType(ii))
	{
	  case OUTPUTVALUES:
	    theData = outputData;
	    break;
	  case COMPUTEDVALUES:
	    theData = computedData;
	    break;
	  case FIXEDVALUES:
	    theData = fixedData;
	    break;
	  default:
	    break;
	}

      graph->currentSettings.clearDataSet(ii);
      xCol = graph->currentSettings.dataSetXCol(ii) - 1;
      yCol = graph->currentSettings.dataSetYCol(ii) - 1;

      if(xCol >= theData.count() || yCol >= theData.count()) continue;

      double xAxisMin  = graph->axisMinX(ii);  // values from graph settings
      double xAxisMax  = graph->axisMaxX(ii);  // (i.e. regardless of zooming)
      double axisScope = xAxisMax - xAxisMin;
      double dataScope = theData[xCol].maxVal() - theData[xCol].minVal();

      double yAxisMin  = -0.1*DBL_MAX;
      double yAxisMax  =  0.1*DBL_MAX;
      if (graph->currentSettings.axisLogarithmic(graph->currentSettings.dataSetYAxis(ii)))
	yAxisMin = MIN_LOG_VAL;

      if( !graph->hasAutoScaleX(ii) && dataScope > 2 * axisScope )
	{
	  for(uint jj = 0; jj < theData[xCol].size(); jj++)
	    {
	      bool xOk = (theData[xCol][jj] >= xAxisMin) && (theData[xCol][jj] <= xAxisMax);
	      bool yOk = (theData[yCol][jj] >= yAxisMin) && (theData[yCol][jj] <= yAxisMax);
	      if(xOk && yOk)
		{
		  graph->currentSettings.dataSetXAppend(ii, theData[xCol][jj]);
		  graph->currentSettings.dataSetYAppend(ii, theData[yCol][jj]);
		}
	      else if (xOk)
		{
		  uint lastsize = graph->currentSettings.dataSetSize(ii);
		  if (lastsize &&
		      isnan((graph->currentSettings.dataSetYData(ii))[lastsize-1])) continue;

		  graph->currentSettings.dataSetXAppend(ii, theData[xCol][jj]);
		  graph->currentSettings.dataSetYAppend(ii, NAN);
		}
	    }
	}
      else
	{
	  for(uint jj = 0; jj < theData[xCol].size(); jj++)
	    {
	      bool yOk = (theData[yCol][jj] >= yAxisMin) && (theData[yCol][jj] <= yAxisMax);

	      if(yOk)
		{
		  graph->currentSettings.dataSetXAppend(ii, theData[xCol][jj]);
		  graph->currentSettings.dataSetYAppend(ii, theData[yCol][jj]);
		}
	      else
		{
		  uint lastsize = graph->currentSettings.dataSetSize(ii);
		  if (lastsize &&
		      isnan((graph->currentSettings.dataSetYData(ii))[lastsize-1])) continue;

		  graph->currentSettings.dataSetXAppend(ii, theData[xCol][jj]);
		  graph->currentSettings.dataSetYAppend(ii, NAN);
		}
	    }
	}
    }

  updateGraph(graph, redrawType);
}

void EBTtool::destroy()
{
  if(errorMessage) delete errorMessage;
  if (myArgv)
    {
      for(int ii = 0; ii < myArgc; ++ii)
	delete [] myArgv[ii];
      delete [] myArgv;
    }
  myArgc = 0;
  myArgv = NULL;

  TextEditor *te;
  while(teList->size() > 0)
    {
      te = teList->first();
      te->parentClose();
    }
  EBTGraphForm *graph;
  while(ogList->size() > 0)
    {
      graph = ogList->first();
      graph->parentClose();
    }
  while(sgList->size() > 0)
    {
      graph = sgList->first();
      graph->parentClose();
    }

  delete teList;
  delete ogList;
  delete sgList;
  delete timer;
}

bool EBTtool::modelCompile(bool interactive)
{
  int ok = 1;
  QLibrary lbn;

  lbn.setFileName(model.soName());

  if(interactive || !lbn.load())
    {
      CompileForm *compileForm = new CompileForm(this, TRUE);
      compileForm->setModel(model.absFilePath());
      ok = compileForm->exec();
      delete compileForm;
    }
  if (lbn.isLoaded()) lbn.unload();

  if (ok) return TRUE;
  else return FALSE;
}

void EBTtool::getOutFileSize()
{
    if(!outFile.fileName().isEmpty())
  outFile.fillNums();
    updateView();
}

void EBTtool::clearData(int clearMemory, int clearFiles)
{
  if(!clearMemory && !clearFiles)
    return;

  if(clearFiles & OUTPUTVALUES)
    {
      if(QFile(outFile.absFilePath()).exists())
	QFile(outFile.absFilePath()).remove();
      if(QFile(repFile.absFilePath()).exists())
	QFile(repFile.absFilePath()).remove();
      if(QFile(esfFile.absFilePath()).exists())
	QFile(esfFile.absFilePath()).remove();

      // The following files are generated by the bifurcation code
      QString bifName = run.absFilePath() + ".avg.out";
      if(QFile(bifName).exists()) QFile(bifName).remove();

      bifName = run.absFilePath() + ".gavg.out";
      if(QFile(bifName).exists()) QFile(bifName).remove();

      bifName = run.absFilePath() + ".max.out";
      if(QFile(bifName).exists()) QFile(bifName).remove();

      bifName = run.absFilePath() + ".min.out";
      if(QFile(bifName).exists()) QFile(bifName).remove();

      bifName = run.absFilePath() + ".minmax.out";
      if(QFile(bifName).exists()) QFile(bifName).remove();

      bifName = run.absFilePath() + ".var.out";
      if(QFile(bifName).exists()) QFile(bifName).remove();
    }
  if(clearFiles & COMPUTEDVALUES)
    {
      if(QFile(compFile.absFilePath()).exists())
	QFile(compFile.absFilePath()).remove();
    }
  if(clearFiles & FIXEDVALUES)
    {
      if(QFile(fixFile.absFilePath()).exists())
	QFile(fixFile.absFilePath()).remove();
    }
  if(clearFiles & STATEOUTPUT)
    {
      QString csbName = run.absFilePath() + "." + csbFile.fileExtension();
      if(QFile(csbName).exists())
	QFile(csbName).remove();
    }

  if(clearMemory & OUTPUTVALUES)
    outputData.clear();
  if(clearMemory & COMPUTEDVALUES)
    computedData.clear();
  if(clearMemory & FIXEDVALUES)
    fixedData.clear();

  EBTGraphForm *graph;
  for (int gg = 0; gg < ogList->size(); ++gg)
    {
      graph = ogList->at(gg);

      for (uint ii = 0; ii < graph->currentSettings.dataSetMapSize(); ii++)
	if(clearMemory & graph->currentSettings.dataSetDataType(ii))
	  graph->currentSettings.clearDataSet(ii);
      updateGraph(graph, REDRAWPLOT);
    }

  if(clearMemory & STATEOUTPUT)
    {
      for (int gg = 0; gg < sgList->size(); ++gg)
	{
	  graph = sgList->at(gg);

	  for (uint ii = 0; ii < graph->currentSettings.dataSetMapSize(); ii++)
	    graph->currentSettings.clearDataSet(ii);
	  updateGraph(graph, REDRAWPLOT);
	}
      csbFile.unload();
      csbFile.setFileName("");      //???????????????
    }

  updateMenu(FALSE);
  // outputRedraw = REDRAWPLOT;
  // updateOutputGraphs();
}

void EBTtool::disableChildWidgets(bool disable)
{
  for (int gg = 0; gg < teList->size(); ++gg)
    teList->at(gg)->setDisabled(disable);

  for (int gg = 0; gg < ogList->size(); ++gg)
    ogList->at(gg)->setDisabled(disable);

  for (int gg = 0; gg < sgList->size(); ++gg)
    sgList->at(gg)->setDisabled(disable);
}

void EBTtool::graphHides(EBTGraphForm *graph)
{
  QCoreApplication::processEvents (QEventLoop::AllEvents, 200);
#ifdef DEBUG
  qDebug("EBTtool::graphHides()");
#endif
  if(!isHidden() && !isMinimized())
    {
      graph->setHideFlag(TRUE);
      graph->hide();
      updateGraphMenu();
#ifdef DEBUG
      qDebug("Hiding graph");
#endif
    }
  QCoreApplication::processEvents (QEventLoop::AllEvents);
}

void EBTtool::graphCloses(EBTGraphForm *graph)
{
  if(!ogList->removeAll(graph))
    sgList->removeAll(graph);

  if (graph->getFile()) delete graph->getFile();

  graph->deleteLater();
  delete graph;
#ifdef DEBUG
  qDebug("Closing graph via deleteLater()");
#endif

  updateGraphMenu();
}

void EBTtool::editorCloses(TextEditor *te)
{
  teList->removeAll(te);
  te->deleteLater();
  delete te;
}


bool EBTtool::processErrors(QString displayString, int errNo)
{
  bool fatal = FALSE;

  if(errNo & WARNING)
    {
      QMessageBox::warning(this, QString("Warning"), QString("Warning in %1:\n%2")
	  .arg(displayString)
	  .arg(model.errorMsg()));
      fatal = FALSE;
    }
  if(errNo & NON_FATAL_ERROR)
    {
      QMessageBox::warning(this, QString("Warning"), QString("Non-fatal error in %1:\n%2")
	    .arg(displayString)
	    .arg(model.errorMsg()));
      fatal = FALSE;
    }
  if(errNo & FATAL_ERROR)
    {
      QMessageBox::critical(this, QString("Error"), QString("Fatal error in %1:\n%2")
	  .arg(displayString)
	  .arg(model.errorMsg()));
      fatal = TRUE;
    }
  if(errNo & LIB_NOT_LOADED)
    {
      QMessageBox::critical(this, QString("Error"), QString("No model library loaded"));
      fatal = FALSE;
    }
  if(errNo & FPE_ERROR)
    {
      QMessageBox::critical(this, QString("Error"), QString("Floating point exception occurred in %1")
	  .arg(displayString));
      fatal = TRUE;
    }

  return fatal;
}

void EBTtool::processFPEerrors(QString routineName, int errNo )
{
  QString displayMsg;

#ifdef DEBUG
  qDebug() <<  "Entering EBTtool::processFPEerrors";
#endif
#if (defined(Q_OS_LINUX) || defined(Q_OS_MACOS)  || (defined(Q_OS_WIN) && defined(__MINGW32__)))
  if(errNo & FE_DIVBYZERO)
    displayMsg = routineName + ":\nDivision by zero";
  if(errNo & FE_INVALID)
    displayMsg = routineName + ":\nInvalid operation";
  if(errNo & FE_OVERFLOW)
    displayMsg = routineName + ":\nResults not representable due to overflow";

#elif defined(Q_OS_WIN)
  if(errNo & EM_ZERODIVIDE)
    displayMsg = routineName + ":\nDivision by zero";
  if(errNo & EM_INVALID)
    displayMsg = routineName + ":\nInvalid operation";
  if(errNo & EM_OVERFLOW)
    displayMsg = routineName + ":\nResults not representable due to overflow";
#endif
  processErrors(displayMsg, FPE_ERROR);
}


void EBTtool::projectEdit()
{
  ProjectForm *pform =  new ProjectForm(this, TRUE);
  pform->setProject(currentProject);
  pform->exec();
  delete pform;

  //    statusBar()->showMessage("Ready");
}


void EBTtool::catchSignal(int signl)
{
    model.handleSignal(signl);
}

void EBTtool::updateProject()
{
    if(!model.fileName().isEmpty())
  currentProject.addFile(EBTMODEL, model.absFilePath());
    if(!run.fileName().isEmpty())
  currentProject.addFile(EBTRUN, run.absFilePath());
    currentProject.setArguments(runArgs);
    currentProject.setSaveTime(QDateTime::currentDateTime().toString(Qt::TextDate));
    EBTGraphForm *graph;
    for (int gg = 0; gg < ogList->size(); ++gg)
      {
  graph = ogList->at(gg);
  if(graph->fileName().isEmpty())
  {
      QMessageBox::warning(0, QString(EBTCAPTION + "unnamed graph"), QString("First save unnamed graph please."));
      graph->fileSaveAs();
  }
  if(!graph->fileName().isEmpty())
    {
      uint filerefnr = currentProject.addFile(EBTOGR, graph->absFilePath());
      currentProject.setFileRefXPos(filerefnr, (graph->pos()).x());
      currentProject.setFileRefYPos(filerefnr, (graph->pos()).y());
      currentProject.setFileRefWidth(filerefnr, (graph->size()).width());
      currentProject.setFileRefHeight(filerefnr, (graph->size()).height());
    }
    }
    for (int gg = 0; gg < sgList->size(); ++gg)
      {
  graph = sgList->at(gg);
  if(graph->fileName().isEmpty())
    {
      QMessageBox::warning(0, QString(EBTCAPTION + "unnamed graph"), QString("First save unnamed graph please"));
      graph->fileSaveAs();
    }
  if(!graph->fileName().isEmpty())
    {
      uint filerefnr = currentProject.addFile(EBTSGR, graph->absFilePath());
      currentProject.setFileRefXPos(filerefnr, (graph->pos()).x());
      currentProject.setFileRefYPos(filerefnr, (graph->pos()).y());
      currentProject.setFileRefWidth(filerefnr, (graph->size()).width());
      currentProject.setFileRefHeight(filerefnr, (graph->size()).height());
    }
      }
}


void EBTtool::projectClose()
{
    if(!okToClear())
  return;
    csbFile.unload();
    csbFile.setFileName("");
    fileNew(ebtprFile);
    model.setFileName(QString()); // is de basename!
    run.setFileName(QString()); // is de basename!
    outFile.setFileName(QString());
    csbFile.setFileName(QString());
    defidcard.setFileName(QString());
    defout.setFileName(QString());
    runArgs = QString();
    clearData(OUTPUTVALUES | COMPUTEDVALUES | FIXEDVALUES | STATEOUTPUT, 0);
    updateView();
    updateMenu(FALSE);
}


void EBTtool::stateOutputBrowse(uint redrawType)
{
  bool          colError = FALSE;
  bool          popError = FALSE;
  uint          xCol, yCol, popNr = 0;
  EBTGraphForm *graph;

  if (!csbFile.checkFile())
    {
      for (int gg = 0; gg < sgList->size(); ++gg) updateGraph(sgList->at(gg), REDRAWPLOT);
      return;
    }
  if (csbFile.fileChanged())
    {
      disconnect(timer, SIGNAL(timeout()), 0, 0);
      timer->stop();
      QMouseEvent event(QEvent::MouseButtonRelease, QPointF(0, 0), QPointF(0, 0), Qt::NoButton, Qt::NoButton, Qt::NoModifier);
      qApp->sendEvent(stateBrowseScrollBar, &event);
      QMessageBox::critical(this, QString(EBTCAPTION + "File changed"), QString("File %1 has changed. Open the file anew").arg(csbFile.fileName()));
      qApp->sendEvent(stateBrowseScrollBar, &event);
    }
  else                                                                               // csb file still ok
    {
      for (int gg = 0; gg < sgList->size(); ++gg)
        {
          graph = sgList->at(gg);
          for (uint ii = 0; ii < graph->currentSettings.dataSetMapSize(); ii++)
            {
              graph->currentSettings.clearDataSet(ii);
              popNr = graph->currentSettings.dataSetPopulationNr(ii);
              xCol  = graph->currentSettings.dataSetXCol(ii) - 1;
              yCol  = graph->currentSettings.dataSetYCol(ii) - 1;
              if (xCol >= csbFile.numColumns(currentStateRecord, popNr) || yCol >= csbFile.numColumns(currentStateRecord, popNr) ||
                  popNr >= csbFile.numPopulations(currentStateRecord))
                {
                  if (popNr >= csbFile.numPopulations(currentStateRecord))
                    {
                      popError = TRUE;
                      continue;
                    }
                  if (xCol >= csbFile.numColumns(currentStateRecord, popNr) || yCol >= csbFile.numColumns(currentStateRecord, popNr)) colError = TRUE;
                  continue;
                }
              double *colPtr[2];
              csbFile.column(currentStateRecord, popNr, xCol, yCol, colPtr);
              for (uint cohortNr = 0; cohortNr < csbFile.numCohorts(currentStateRecord, popNr); cohortNr++)
                {
                  graph->currentSettings.dataSetXAppend(ii, colPtr[0][cohortNr]);
                  graph->currentSettings.dataSetYAppend(ii, colPtr[1][cohortNr]);
                }
              csbFile.clearColumnData();
            }                                                                        // for each dataSet
          if (colError)
            {
              errorMessage->showMessage(colWarning);
              colError = FALSE;
            }
          if (popError)
            {
              errorMessage->showMessage(popWarning);
              popError = FALSE;
            }
          if (!csbFile.description(currentStateRecord, popNr).isEmpty())
            graph->showText(csbFile.description(currentStateRecord, popNr));
          else
            graph->showText(QString("State graph at t=%1").arg(formatDouble(csbFile.timeVal(currentStateRecord))));

          updateGraph(graph, redrawType);
        }                                                                            // for each graph
    }                                                                                // csb file still ok
  updateView();
  updateMenu(FALSE);
}


void EBTtool::stateBrowsePause()
{
    statePause->setDisabled(TRUE);
    timer->stop();
}


void EBTtool::stateBrowsePlayF()
{
    statePause->setDisabled(FALSE);
    disconnect(timer, SIGNAL(timeout()), 0, 0);
    connect(timer, SIGNAL(timeout()), this, SLOT(stateOutputBrowseForward()));
    int interval = (104 - (10 * stateBrowseComboBox->currentText().toInt())) * 16;
    timer->setSingleShot(FALSE);			// Qt4
    timer->start(interval);
}


void EBTtool::stateBrowsePlayB()
{
    statePause->setDisabled(FALSE);
    disconnect(timer, SIGNAL(timeout()), 0, 0);
    connect(timer, SIGNAL(timeout()), this, SLOT(stateOutputBrowseBackward()));
    int interval = (104 - (10 * stateBrowseComboBox->currentText().toInt())) * 16;
    timer->setSingleShot(FALSE);			// Qt4
    timer->start(interval);
}


void EBTtool::stateOutputBrowseSlot( int stateNr )
{
    currentStateRecord = stateNr - 1;
    stateOutputBrowse(REDRAWCURVES);
}

void EBTtool::stateOutputBrowseForward()
{
    if(currentStateRecord < (int)(csbFile.numStates() - 1))
//  currentStateRecord++;
  stateBrowseScrollBar->setValue(currentStateRecord + 2);
    else
    {
  disconnect(timer, SIGNAL(timeout()), 0, 0);
  timer->stop();
    }
}

void EBTtool::stateOutputBrowseBackward()
{
    if(currentStateRecord > 0)
//  currentStateRecord--;
  stateBrowseScrollBar->setValue(currentStateRecord);
    else
    {
  disconnect(timer, SIGNAL(timeout()), 0, 0);
  timer->stop();
    }
}


void EBTtool::stateBrowseSpeedSlot(const QString& speed)
{
  if(timer->isActive())
    {
      int interval = (104 - (10 * speed.toInt())) * 16;
      timer->setInterval(interval);
    }
}


void EBTtool::stateExport()
{
    EBTTextFile stateExportFile;
    stateExportFile.createNew();
    stateExportFile.setFileExtension(run.isfFile.fileExtension());
    stateExportFile.setDescriptionText(run.isfFile.descriptionText());
    stateExportFile.setSaveText(run.isfFile.descriptionText() + " (*." + run.isfFile.fileExtension() + ")") ;
    stateExportFile.setOpenText(stateExportFile.saveText());

    if(!stateBrowseScrollBar->isHidden())
  stateExportFile.buffer = csbFile.exportState(currentStateRecord);
    else if(model.libLoaded())
  stateExportFile.buffer = model.exportState();
    fileSaveAs(stateExportFile);
}


void EBTtool::loadSettings()
{
    bool ok;
    QSettings settings(QCoreApplication::organizationName(), QCoreApplication::applicationName());

    ok = settings.contains("EBTDir");
    if (ok) EBTDir = settings.value("EBTDir").toString();
    else
      EBTDir = qApp->applicationDirPath();

    settings.beginGroup("MainWindow");
    ok = settings.contains("size");
    if (ok) resize(settings.value("size").toSize());
    move(settings.value("pos", QPoint(200, 200)).toPoint());
    settings.endGroup();
}


void EBTtool::saveSettings()
{
    QSettings settings(QCoreApplication::organizationName(), QCoreApplication::applicationName());

    settings.setValue("EBTDir", EBTDir);
    settings.beginGroup("MainWindow");
    settings.setValue("size", size());
    settings.setValue("pos", pos());
    settings.endGroup();
}


void EBTtool::modelDefIDLoad()
{
  if(!defidcard.libLoaded())
    {
      bool ok = TRUE;
      //  if(!defidcard.libChosen())
      //  {
      if (!defidcard.checkLib())    // c file is chosen, and no up-to-date compiled lib exists yet
	{
	  CompileForm *compileForm = new CompileForm(this, TRUE);
	  compileForm->setDefIDcard(defidcard.absFilePath());
	  ok = compileForm->exec();
	  delete compileForm;
	}
      //  }
      if (ok) defidcard.initHandles();

      if (!csbFile.fileName().isEmpty())
	csbFile.setDefID(&defidcard);

      if (csbFile.scanSucceeded())
	stateOutputBrowse(REDRAWPLOT);
      }
    else
      {
	defidcard.libUnload();
	if(!csbFile.fileName().isEmpty()) csbFile.setDefID(&defidcard);
      }
  updateView();
  updateMenu(FALSE);
}


void EBTtool::modelDefOutLoad()
{
  if(!defout.libLoaded())
    {
      bool ok = TRUE;
      if(!defout.checkLib())  // c file is chosen, and no up-to-date compiled lib exists yet
	{
	  CompileForm *compileForm = new CompileForm(this, TRUE);
	  compileForm->setDefOutput(defout.absFilePath());
	  ok = compileForm->exec();
	  delete compileForm;
	}
  if (ok) defout.initHandles();

  if(ok && !csbFile.fileName().isEmpty())
    {
      if(computedData.count() > 0)
        {
	  ok = (QMessageBox::question( this, EBTCAPTION + "Clear computed",
				      QString("Are you sure you want to delete\nthe current computed data?"),
				      QMessageBox::Yes|QMessageBox::No,
				      QMessageBox::No) == QMessageBox::Yes);

	  if (ok) computedData.clear();
        }

      if (ok)
        {
	  csbFile.setDefOut(&defout);
	  fillComputedData();
        }
      }
    }
  else
    {
      defout.libUnload();
      if(!csbFile.fileName().isEmpty()) csbFile.setDefOut(&defout);
    }
  updateView();
  updateMenu(FALSE);
}


void EBTtool::fillComputedData()
{
    for(uint colNr = 0; colNr < csbFile.outputVarNum(); colNr++)
      {
  EBTData dataVec;
  computedData[colNr] = dataVec;
  for (uint stateNr = 0; stateNr < csbFile.numStates(); stateNr++)
    computedData[colNr].appendValue(csbFile.compOutput(stateNr)[colNr]);
      }
    for (int gg = 0; gg < ogList->size(); ++gg)
      updateOutputGraph(ogList->at(gg), REDRAWPLOT);
}


void EBTtool::fileComputedValuesSave()
{
    if(computedData.count() == 0)
    {
  QMessageBox::warning(this, QString(EBTCAPTION + "no data"), QString("No computed values to be saved"));
  return;
    }
    EBTTextFile compDataFile;
    compDataFile.createNew();
    compDataFile.setFileExtension(compFile.fileExtension());
    compDataFile.setDescriptionText(compFile.descriptionText());
    compDataFile.setSaveText(compDataFile.descriptionText() + " (*." + compDataFile.fileExtension() + ");;Any files (*)");
    compDataFile.setOpenText(compDataFile.saveText());
    QString outputString;
    for (uint ii = 0; ii < computedData[0].size(); ii++)
    {
  for (int jj = 0; jj < computedData.count(); jj++)
  {
      if(jj > 0)
    outputString += "\t";
      outputString += formatDouble(computedData[jj][ii]);
  }
  outputString += "\n";
    }
    compDataFile.buffer = outputString;
    fileSaveAs(compDataFile);
}


void EBTtool::checkEBTDir()
{
    if(!QDir(EBTDir + "/integrator").exists() || !QDir(EBTDir + "/templates").exists())
      EBTDir = qApp->applicationDirPath();

    if(!QDir(EBTDir + "/integrator").exists() || !QDir(EBTDir + "/templates").exists())
      {
        QProcessEnvironment toolenv = QProcessEnvironment::systemEnvironment();
        if (toolenv.contains(QString("EBTPATH")))
          EBTDir = toolenv.value(QString("EBTPATH"));
      }

#if defined(Q_OS_MACOS) && !defined(DEBUG)
    EBTDir.replace(QString("/Contents/MacOS"), QString(), Qt::CaseInsensitive);
    EBTDir.append(QString("/Contents/Resources"));
    if(!QDir(EBTDir + "/integrator").exists() || !QDir(EBTDir + "/templates").exists())
      {
	      CFURLRef 	appUrlRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
	      CFStringRef 	macPath   = CFURLCopyFileSystemPath(appUrlRef, kCFURLPOSIXPathStyle);

	      EBTDir = QString(CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding()));
	      EBTDir.replace(QString("/Contents/MacOS"), QString(), Qt::CaseInsensitive);
	      EBTDir.append(QString("/Contents/Resources"));

	      CFRelease(appUrlRef);
	      CFRelease(macPath);
      }
#endif

    if(!QDir(EBTDir + "/integrator").exists() || !QDir(EBTDir + "/templates").exists() )
      {
	      QMessageBox::critical(this, QString(EBTCAPTION + "EBT directory"),
	      QString("EBTtool can't find it's template and integrator files in:\n\n" + EBTDir + "\n\nPlease set EBT directory correctly."));
    	  QString dName = QFileDialog::getExistingDirectory(this, "Choose the path to where EBT is installed", EBTDir, QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
	      if (!dName.isEmpty()) EBTDir = dName;

	      if(!QDir(EBTDir + "/integrator").exists() || !QDir(EBTDir + "/templates").exists())
	        {
	          QMessageBox::critical(this, QString(EBTCAPTION + "EBT dir"), QString("EBT dir is still not correct. This will cause trouble when creating or compiling models."));
	          return;
	        }
      }

#if (defined(Q_OS_WIN) && defined(__MINGW32__))
    if (QDir(EBTDir + "/mingw/bin").exists())
      {
	      QByteArray pathVal = EBTDir.toLatin1();
	      pathVal.append("/mingw/bin");
	      qputenv("PATH", pathVal);
      }
    else if (QDir(qApp->applicationDirPath() + "/mingw/bin").exists())
      {
	      QByteArray pathVal = (qApp->applicationDirPath()).toLatin1();
	      pathVal.append("/mingw/bin");
	      qputenv("PATH", pathVal);
      }
    else
      QMessageBox::critical(this, QString(EBTCAPTION + "EBT directory"),
	  QString("EBTtool can't find the included MinGW compiler directory in:\n\n" + EBTDir + "\n\nor in:\n\n" + qApp->applicationDirPath() + "\n\nPlease set EBT directory correctly."));
#endif

#ifdef DEBUG
    qDebug() <<  "Using EBT library files from " << EBTDir;
#endif

    saveSettings();    // other parts of the program will also look for EBTDir in the registry/settings
}


void EBTtool::modelUnload()
{
  if(model.libLoaded()) model.libUnload();
}


void EBTtool::initVars()
{
  myArgv = NULL;
  c_model = NULL;
  c_run = NULL;
  charRunArg = NULL;

  outputRedraw = REDRAWPLOT;
  stateRedraw   = REDRAWPLOT;
  myArgc = 0;
  runArgs = QString();
  runExtend = FALSE;
  outputGraphDirPath = QString();
  stateGraphDirPath = QString();
  currentStateRecord = 0;

  teList = new QList<TextEditor *>;
  ogList = new QList<EBTGraphForm *>;
  sgList = new QList<EBTGraphForm *>;
  timer  = new QTimer(this);
}


