/***
   NAME
     ebttool - header file
   DESCRIPTION
     
   Last modification: AMdR - Feb 01, 2007
***/
#ifndef EBTTOOL
#define EBTTOOL
#include <QMainWindow>

#include "ui_ebttool.h"   // defines Ui::EBTtool
#include "ComputeThread.h"

class EBTtool : public QMainWindow, public Ui::EBTtool
{
    Q_OBJECT

public:
    EBTtool(QWidget* parent = 0, Qt::WindowFlags fl = Qt::Window);
    ~EBTtool();

    virtual void fileNew( EBTFile & ef );
    virtual void fileOpen( EBTFile & ef );
    virtual void fileSaveAs( EBTFile & ef );
    virtual QString fileCopyAs( EBTFile & ef , const QString & nname);
    virtual bool okToClear();
    virtual void loadOutputValues( uint appendData );
    virtual void loadComputedValues( uint appendData );
    virtual void loadFixedValues( uint appendData );
    virtual void loadProject();
    virtual void initFiles();
    virtual bool modelCompile( bool interactive );
    virtual void updateProject();
    virtual void stateOutputBrowse( uint redrawType );
    virtual void loadSettings();
    virtual void saveSettings();
    virtual void checkEBTDir();
    virtual void initVars();

signals:
    void setReady(bool ready);

public slots:
    virtual void fileExit();
    virtual void helpIndex();
    virtual void helpContents();
    virtual void helpAbout();
    virtual void helpAboutQt();
    virtual void fileOutputValuesOpen();
    virtual void fileComputedValuesOpen();
    virtual void fileFixedValuesOpen();
    virtual void fileRereadValues();
    virtual void graphNewGraphSlot( bool outputGraph );
    virtual void graphOutputGraphOpen();
    virtual void graphOutputGraphNew();
    virtual void graphOutputGraph( EBTOGrFile & graphFile );
    virtual void graphOutputGraph( EBTOGrFile & graphFile, int xp, int yp, int wd, int hg );
    virtual void graphStateGraphOpen();
    virtual void graphStateGraphNew();
    virtual void graphStateGraph( EBTSGrFile & graphFile );
    virtual void graphStateGraph( EBTSGrFile & graphFile, int xp, int yp, int wd, int hg );
    virtual void modelCompileSlot();
    virtual void modelDefIDEdit();
    virtual void modelDefIDOpen();
    virtual void modelDefIDNew();
    virtual void modelDefIDClean();
    virtual void modelDefOutEdit();
    virtual void modelDefOutOpen();
    virtual void modelDefOutNew();
    virtual void modelDefOutClean();
    virtual void modelHeaderEdit();
    virtual void modelProgramEdit();
    virtual void modelAllCopy();
    virtual void modelHeaderCopy();
    virtual void modelProgramCopy();
    virtual void modelOpen();
    virtual bool processErrors( QString displayString, int errNo );
    virtual void processFPEerrors( QString routineName, int errNo );
    virtual void runDataClear();
    virtual void runAllCopy();
    virtual void runCVFCopy();
    virtual void runISFCopy();
    virtual void runESFCopyToISF();
    virtual void runCVFEdit();
    virtual void runISFEdit();
    virtual void runExtendSlot();
    virtual void runExecuteSlot();
    virtual void runEnded();
    virtual void runTogglePause();
    virtual void runStop();
    virtual void runAllNew();
    virtual void runCVFNew();
    virtual void runISFNew();
    virtual void runOpen();
    virtual void fileStateOutputOpen();
    virtual void updateView();
    virtual void projectSave();
    virtual void projectSaveAs();
    virtual void closeEvent( QCloseEvent * );
    virtual void fileEdit( EBTFile & ef, bool viewOnly );
    virtual void projectNew();
    virtual void modelNew();
    virtual void updateOutputGraphs();
    virtual void updateStateGraphs();
    virtual void updateGraph( EBTGraphForm * graph, int redrawType );
    virtual void projectOpen();
    virtual void runCVFView();
    virtual void runISFView();
    virtual void runESFView();
    virtual void runRepView();
    virtual void runOutputValuesView();
    virtual void runArgumentsEdit();
    virtual void updateGraphSlot( EBTGraphForm * graph, bool isOutputGraph );
    virtual void updateOutputGraph( EBTGraphForm * graph, uint redrawType );
    virtual void getOutFileSize();
    virtual void clearData( int clearMemory, int clearFiles );
    virtual void disableChildWidgets( bool disable );
    virtual void graphCloses( EBTGraphForm * graph );
    virtual void graphHides( EBTGraphForm * graph );
    virtual void editorCloses( TextEditor * te );
    virtual void projectEdit();
    virtual void catchSignal( int signl );
    virtual void projectClose();
    virtual void stateBrowsePause();
    virtual void stateBrowsePlayF();
    virtual void stateBrowsePlayB();
    virtual void stateOutputBrowseSlot( int stateNr );
    virtual void stateOutputBrowseForward();
    virtual void stateOutputBrowseBackward();
    virtual void stateBrowseSpeedSlot( const QString & speed );
    virtual void stateExport();
    virtual void modelDefIDLoad();
    virtual void modelDefOutLoad();
    virtual void fileComputedValuesSave();
    virtual void modelUnload();
    virtual void updateGraphMenu();

protected slots:
    virtual void languageChange();

protected:
    void hideEvent ( QHideEvent * );
    void showEvent ( QShowEvent * );

private:
    QString oldDir;
    QString EBTDir;
    EBTDataMap fixedData;
    EBTDataMap computedData;
    uint outputRedraw;
    EBTRun run;
    char *charRunArg;
    char **myArgv;
    char *c_run;
    QList<EBTGraphForm *> *sgList;
    EBTProject currentProject;
    QList<TextEditor *> *teList;
    QString runArgs;
    QList<EBTGraphForm *> *ogList;
    EBTDataMap outputData;
    bool haveABreak;
    bool runExtend;
    ComputeThread thread;
    EBTModel model;
    char *c_model;
    int myArgc;
    QErrorMessage *errorMessage;
    int currentStateRecord;
    QTimer *timer;
    EBTDefIDcard defidcard;
    EBTDefOut defout;
    QString outputGraphDirPath;
    QString stateGraphDirPath;
    uint stateRedraw;

    void init();
    virtual bool modelInit();
    virtual void updateMenu( bool allDisabled );
    virtual void destroy();
    virtual void fillComputedData();

    void hideCSBControls( bool allHidden );
    void disableCSBControls( bool allHidden );
};


#endif /* EBTTOOL */
