#include <qfileinfo.h>
#include <QTextStream>
#include <qmessagebox.h>
#include <qpen.h>
#include "ebtxmlfile.h"

EBTXMLFile::EBTXMLFile()
{
}

EBTXMLFile::~EBTXMLFile()
{
}

void EBTXMLFile::load()
{
    loadSettings();
    
    QString fName;
    if(!fileName().isEmpty())
	fName = absFilePath();
    else
	fName = EBTDir + "/" + templateFile();
    
    QFile xmlFile(fName);
    if (!xmlFile.open(QIODevice::ReadOnly | QIODevice::Text))
    {
	QMessageBox::critical(0, EBTCAPTION + "Can't open",
			      QString("Cannot open file %1").arg(fName));
	return;
    }
    if (!domTree.setContent(&xmlFile))
    {
	QMessageBox::critical(0, EBTCAPTION + "Can't parse",
			      QString("Parsing error for file %1").arg(fName));
	return;
    }
    xmlFile.close();
    QDomElement root = domTree.documentElement();
    QDomNode node = root.firstChild();
    
    while ( !node.isNull() ) 
    {
	QDomElement element = node.toElement();
	if(!element.isNull())
	    readElement(element); // specific for exact type of file
	node = node.nextSibling();
    }
    //    setChanged(FALSE);
}

bool EBTXMLFile::save()
{
    QString fName;
    if (fileName().isEmpty())
	return saveAs();
    if (!fileName().isEmpty())
	fName = absFilePath();
    
    QFile file(fName);
    
    QDomDocument domDoc = createDomDoc("ebtxml");
    
    if (file.open(QIODevice::WriteOnly))
    {
	QTextStream stream(&file);
	stream << domDoc.toString();
	file.close();
	setChanged(FALSE);
	return TRUE;
    }
    else
    {
	QMessageBox::critical(0, EBTCAPTION + "Save failed",
			       QString("Failed to save '%1'").
			       arg(fName));  // willen we dit eigenlijk wel?
	return FALSE;
    }
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const QString &value)
{
    QDomAttr attr = doc.createAttribute(name);
    attr.setValue(value);
    node.setAttributeNode(attr);
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const QFont &value)
{
    addAttribute(doc, node, QString(name + ".family"), value.family());
    addAttribute(doc, node, QString(name + ".size"), value.pointSize());
    addAttribute(doc, node, QString(name + ".weight"), value.weight());
    addAttribute(doc, node, QString(name + ".italic"), value.italic());
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const int &value)
{
    QString str;
    addAttribute(doc, node, name, str.setNum(value));
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const double &value)
{
    QString str;
    addAttribute(doc, node, name, str.setNum(value));
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const QPen &value)
{
    addAttribute(doc, node, QString(name + ".width"), (int)value.width());
    addAttribute(doc, node, QString(name + ".color"), value.color());
    addAttribute(doc, node, QString(name + ".style"), (int)value.style());
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const QwtSymbol *value)
{
    addAttribute(doc, node, QString(name + ".style"), (int)value->style());
    addAttribute(doc, node, QString(name + ".size"), value->size().height());
    addAttribute(doc, node, QString(name + ".brushstyle"), (int)value->brush().style());
    addAttribute(doc, node, QString(name + ".color"), value->pen().color());
    addAttribute(doc, node, QString(name + ".penwidth"), (int)value->pen().width());
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const QColor &value)
{
    addAttribute(doc, node, QString(name +".red"), value.red());
    addAttribute(doc, node, QString(name +".green"), value.green());
    addAttribute(doc, node, QString(name +".blue"), value.blue());
}

void EBTXMLFile::addAttribute(QDomDocument &doc, QDomElement &node, const QString name, const bool &value)
{
    QString str;
    if ( value )
	str = "true";
    else
	str = "false";
    addAttribute(doc, node, name, str);
}

QString EBTXMLFile::readStringAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    *attributeFound = TRUE;
    if(hasAttribute(el, name))
	return el.attribute(name, "");
    else
	    *attributeFound = FALSE;
    return QString();
}

QFont EBTXMLFile::readFontAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    bool allAttributesFound;
    *attributeFound = TRUE;
    QFont font;
    font.setFamily(readStringAttribute(el, QString(name +".family"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    font.setPointSize(readIntAttribute(el, QString(name + ".size"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    font.setWeight((QFont::Weight)readIntAttribute(el, QString(name + ".weight"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    font.setItalic(readBoolAttribute(el, QString(name + ".italic"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    return font;
}

int EBTXMLFile::readIntAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    return readStringAttribute(el, name, attributeFound).toInt();
}

double EBTXMLFile::readDoubleAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    bool ok;
    double d = readStringAttribute(el, name, attributeFound).toDouble(&ok);
    if (ok)
	return d;
    else
	return 0.0;
}

QPen EBTXMLFile::readPenAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    bool allAttributesFound;
    *attributeFound = TRUE;
    QPen pen;
    pen.setWidth((uint)readIntAttribute(el, QString(name + ".width"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    pen.setColor(readColorAttribute(el, QString(name + ".color"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    pen.setStyle((Qt::PenStyle)readIntAttribute(el, QString(name + ".style"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    return pen;
}

QwtSymbol * EBTXMLFile::readSymbolAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    bool allAttributesFound;
    *attributeFound = TRUE;
    QwtSymbol *symbol = new QwtSymbol();
    symbol->setStyle((QwtSymbol::Style)readIntAttribute(el, QString(name + ".style"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    symbol->setSize(readIntAttribute(el, QString(name + ".size"), &allAttributesFound));
    if(!allAttributesFound) *attributeFound = FALSE;
    symbol->setBrush(QBrush(readColorAttribute(el, QString(name + ".color"), &allAttributesFound),
			   (Qt::BrushStyle)readIntAttribute(el, QString(name + ".brushstyle"))));
    if(!allAttributesFound) *attributeFound = FALSE;
    symbol->setPen(QPen(readColorAttribute(el, QString(name + ".color"), &allAttributesFound), (uint)readIntAttribute(el, QString(name + ".penwidth")), Qt::SolidLine));
    if(!allAttributesFound) *attributeFound = FALSE;
    return symbol;
}

QColor EBTXMLFile::readColorAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    bool allAttributesFound;
    *attributeFound = TRUE;
    int red = readIntAttribute(el, QString(name + ".red"), &allAttributesFound);
    if(!allAttributesFound) *attributeFound = FALSE;
    int green = readIntAttribute(el, QString(name + ".green"), &allAttributesFound);
    if(!allAttributesFound) *attributeFound = FALSE;
    int blue = readIntAttribute(el, QString(name + ".blue"), &allAttributesFound);
    if(!allAttributesFound) *attributeFound = FALSE;
    return QColor(red, green, blue);
}

bool EBTXMLFile::readBoolAttribute(const QDomElement &el, const QString &name, bool *attributeFound)
{
    // bool b;
    //    if (readStringAttribute(el, name) == "false")
//	b = FALSE;
//    else
//	b = TRUE;
    if (readStringAttribute(el, name, attributeFound) == "true")
	return TRUE;
    return FALSE;
}

bool EBTXMLFile::hasAttribute(const QDomElement &el, const QString &name)
{
    return el.hasAttribute(name);
}
