/***
  The routine below is a template that can be freely adapted. The routine can
  be used to compute additional popIDcard (i.e. i_const()) variables for every
  cohort in a population from the EBT state files (CSB file) within the EBTTOOL
  program. Hence, simulating the model anew to obtain interesting cohort
  quantities is not necessary any more if the state of all populations has been
  frequently stored in the CSB file. The routine can be changed, compiled, and
  dynamically linked into the EBTTOOL program.

  The routine DefineIDcard() is structured along EXACTLY the same lines as
  the routines in the EBT program, hence use the identical syntax.

  The values of I_STATE_DIM and NEW_CONST_DIM have to be defined prior to the
  routine DefineIDcard(). I_STATE_DIM should be set to the number of i-state
  variables stored per cohort in the CSB file. The value of NEW_CONST_DIM
  should be set to the number of additional IDcard variables that are computed
  in the DefineIDcard() routine below.

  To address popIDcard variables already stored in the CSB file, use the syntax

			popIDcard[pop. #][cohort #][i_const(#)]

  To address the additional popIDcard variables, defined in the routine below,
  use the syntax:

			newIDcard[pop. #][cohort #][#]

  with the last index ranging from 0 to (NEW_CONST_DIM-1). DO NOT USE THE
  i_const() MACRO IN THIS SYNTAX! Failure to use the syntax as such, may lead
  to corruption of data, incorrect computations or program crashes.

  The integer array "cohort_no[]" is globally available and denotes the number
  of internal cohorts present in each structured population.

  Compilation and linking is completed controlled from within the EBTTOOL
  program.
***/

#define I_STATE_DIM	  2
#define NEW_CONST_DIM	  2


void	DefineIDcard(double *env, population *pop, population *newIDcard)
  
{
  register int		i;

  for(i=0; i<cohort_no[0]; i++)
    {
      newIDcard[0][i][0] = pow(pop[0][i][i_state(1)], 2.0);
      newIDcard[0][i][1] = pow(pop[0][i][i_state(1)], 3.0);
    }
  
  return;
}



/*==========================================================================*/
