#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "math.h"
#include <sys/stat.h>


#define USAGE		"csb2test <CSB filename>"

// List of magic keys of all recognized CSB file types

#define CSB_MAGIC_KEY1		20030509

typedef struct envdim {
                        double          timeval;
                        int             columns;
                        int             data_offset;
                        long            memory_used;
                       } Envdim;

double			csb_pars[100];



/*==============================================================================*/

int			main(int argc, char **argv)

{
  char			*fn;
  FILE			*fp = NULL;
  struct stat		stattmp;
  long			filepos = 0L, magic;
  Envdim		cur_env;
  int			i, pars, file_ok = 1;

  switch (argc)
    {
    case 2:
      fn = argv[1];
      if (!strcmp(fn+strlen(fn)-1, ".")) strcat(fn, "csb");
      else if (strcmp(fn+strlen(fn)-4, ".csb")) strcat(fn, ".csb");

      if (!stat(fn, &stattmp))			/* couldn't stat            */
	{
	  if ((!strcmp(fn+strlen(fn)-3, "csb")) && (stattmp.st_mode & S_IFREG))
	    fp = fopen(fn, "r");		/* open if regular file     */
	  else
	    {
	      fprintf(stderr, "File %s is not a regular CSB file.\n", fn);
	      return 1;
	    }
	}
      if (fp == NULL)
	{
	  fprintf(stderr, "Can't open file %s\n", fn);
	  return 1;
	}
      break;
    default:
      fprintf(stderr, "\nUsage: %s\n\n", USAGE);
      return 1;
    }
  
  if (fread(&magic, sizeof(long), 1, fp) != 1)
    {
      fprintf(stderr, "\nCSB file does not contain any data!\n\n");
      return 1;
    }
  
  switch (magic)
    {
    case CSB_MAGIC_KEY1:
      /*
       * CSB file version dd. May 09, 2003 with array of parameter values at
       * the start of the file
       */
      fprintf(stdout, "Magic key: %ld\n\n", magic);
      file_ok = (fread(&pars, sizeof(int), 1, fp) == 1);
      if (file_ok)
	{
	  fprintf(stdout, "Number of parameters: %d\n\n", pars);
	  file_ok = (fread(csb_pars, sizeof(double), pars, fp) == pars);
	  if (file_ok)
	    for (i=0; i<pars; i++) fprintf(stdout, "Parameter %3d: %12g\n", i, csb_pars[i]);
	}
      if (!file_ok)
	{
	  fprintf(stderr, "\nCSB file seems to be invalid or corrupt!\n\n");
	  return 1;
	}
      filepos = ftell(fp);
      break;
    default:
      fprintf(stdout, "No or unknown magic key found.\n\n");
      fseek(fp, filepos, SEEK_SET);
      break;
    }

  fprintf(stdout, "\nFile position: %ld\n", filepos);
  fprintf(stdout, "Size of Envdim: %d\n\n", sizeof(Envdim));
  
  while (fread(&cur_env, sizeof(Envdim), 1, fp) == 1)
    {
      fseek(fp, filepos, SEEK_SET);
      fprintf(stdout, "\nFile position: %20ld\n", filepos);
      fprintf(stdout, "Time value   : %20.4f\n",  cur_env.timeval);
      fprintf(stdout, "Columns      : %20d\n",    cur_env.columns);
      fprintf(stdout, "Data offset  : %20d\n",    cur_env.data_offset);
      fprintf(stdout, "Memory used  : %20ld\n",   cur_env.memory_used);

      fseek(fp, filepos, SEEK_SET);
      fseek(fp, cur_env.memory_used, SEEK_CUR);
      filepos = ftell(fp);
    }

  return 0;
}

